@@ -2,7 +2,7 @@
                           cclient.cpp  -  description
                              -------------------
     begin                : Sun Sep 30 2001
-    copyright            : (C) 2001-2004 by Mathias Kster
+    copyright            : (C) 2001-2004 by Mathias Küster
     email                : mathen@users.berlios.de
  ***************************************************************************/
 
@@ -15,9 +15,10 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "cclient.h"
+
 #include <stdio.h>
 #include <sys/types.h>
-#include <fcntl.h>
 
 #ifndef WIN32
 #include <stdlib.h>
@@ -27,47 +28,43 @@
 #include <netdb.h>
 #endif
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "core/clist.h"
+#include "dcobject.h"
+#include "cquerymanager.h"
+#include "cdownloadmanager.h"
+#include "cconnectionmanager.h"
+#include "cencrypt.h"
+#include "cmessagehandler.h"
+#include "cconfig.h"
+#include "csearchmanager.h"
+#include "clistenmanager.h"
+#include "dclib-ssl-use.h"
 
-#include <dclib/dcos.h>
-#include <dclib/core/clist.h>
-#include <dclib/dcobject.h>
-#include <dclib/cquerymanager.h>
-#include <dclib/cdownloadmanager.h>
-#include <dclib/cconnectionmanager.h>
-#include <dclib/cencrypt.h>
-#include <dclib/cmessagehandler.h>
-#include <dclib/core/cstringlist.h>
-#include <dclib/cconfig.h>
-#include <dclib/csearchmanager.h>
+// need to decompress zpipe data
+#include "core/czlib.h"
+#include "core/cbytearray.h"
 
-// need to decompress zline data
-#include <dclib/core/czlib.h>
+// for DPRINTF
+#include "dclib.h"
 
-#include "cclient.h"
+// IsPrivateI4() and IsValidI4()
+#include "core/cnetaddr.h"
 
-CClient::CClient()
+CClient::CClient( CString remoteEncoding ) : CDCProto( remoteEncoding )
 {
 	m_pCallback = 0;
 	
-	m_sBuffer          = "";
-	m_sHubName         = "";
-	m_sHubTopic        = "";
-	m_sVersion         = "";
-	m_bHandshake       = TRUE;
-	m_bHandleUserList  = TRUE;
-	m_bHandleSearch    = TRUE;
-	m_bHandleMyinfo    = TRUE;
-	m_bHandleForceMove = TRUE;
-	m_bHandleTransfer  = TRUE;
-	m_bUpdateMyinfo    = FALSE;
-	m_bSendMyinfo      = TRUE;
-	m_bSSLMode         = FALSE;
+	m_bHandshake       = true;
+	m_bHandleUserList  = true;
+	m_bHandleSearch    = true;
+	m_bHandleMyinfo    = true;
+	m_bHandleForceMove = true;
+	m_bHandleTransfer  = true;
+	m_bUpdateMyinfo    = false;
+	m_bSendMyinfo      = true;
+	m_bSSLMode         = false;
 	m_nNickListHandler = 0;
-	m_nOpListHandler   = 0;
-	m_bUsedPassword    = FALSE;
+	m_bUsedPassword    = false;
 
 	// set reconnect parameter
 	m_eReconnectState      = ersNONE;
@@ -75,16 +72,34 @@ CClient::CClient()
 	m_timeReconnectTimeout = 0;
 	m_timeMyinfoTimeout    = 0;
 	
-	// zline handling
-	m_zBuffer.SetSize(0);
-	m_bZMode = FALSE;
+	// initialise settings used since hub supports message cannot be used
+	m_bExtProtocol = false;
+	m_bGotIP       = false;
+	
+	// zpipe handling
+	m_pZLib = 0;
+	m_bZMode = false;
+	
+	// message parser
+	m_pMessageHandler = new CMessageHandler( remoteEncoding );
+
+#if DCLIB_HAS_SSL == 1	
+	if ( CConfig::Instance() )
+	{
+		m_MyInfo.m_bTLSFlag = ( CConfig::Instance()->GetTransferCert().NotEmpty() && CConfig::Instance()->GetTransferKey().NotEmpty() );
+	}
+#endif
 }
 
 CClient::~CClient()
 {
 	SetCallBackFunction(0);
 	
-	m_IPcache.Clear();
+	delete m_pZLib;
+	m_pZLib = 0;
+	
+	delete m_pMessageHandler;
+	m_pMessageHandler = 0;
 }
 
 /** */
@@ -94,29 +109,24 @@ void CClient::ConnectionState( eConnectionState state )
 
 	CMessageConnectionState *Object = new CMessageConnectionState();
 
-	if (!Object)
-	{
-		return;
-	}
-
 	if ( (state == estCONNECTED) ||
 	     (state == estDISCONNECTED) )
 	{
 		// reset all values
-		m_sBuffer           = "";
-		m_bHandshake        = TRUE;
+		m_sBuffer.Empty();
+		m_bHandshake        = true;
 		m_timeMyinfoTimeout = time(0);
 		m_nNickListHandler  = 0;
-		m_nOpListHandler    = 0;
+
+		m_bExtProtocol = false;
+		m_bGotIP       = false;
 
 		m_UserList.Clear();
 		
-		// hub IP may have changed
-		m_IPcache.Clear();
-		
-		m_zBuffer.SetSize(0);
-		m_bZMode = FALSE;
-#ifdef HAVE_SSL
+		m_bZMode = false;
+		delete m_pZLib;
+		m_pZLib = 0;
+#if DCLIB_HAS_SSL == 1
 		m_pMessageSSL.Init();
 #endif
 		// update reconnect state
@@ -124,9 +134,9 @@ void CClient::ConnectionState( eConnectionState state )
 		{
 			UpdateReconnect( ersNONE );
 
-			if ( m_bSSLMode == TRUE )
+			if ( m_bSSLMode && CConfig::Instance() )
 			{
-				if ( ChangeSocketMode(esmSSLCLIENT) == FALSE )
+				if ( ChangeSocketMode( esmSSLCLIENT, CConfig::Instance()->GetTransferCert(), CConfig::Instance()->GetTransferKey() ) == false )
 				{
 					printf("change ssl mode failed\n");
 				}
@@ -174,7 +184,7 @@ void CClient::ConnectionState( eConnectionState state )
 void CClient::DataAvailable( const char * buffer, int len )
 {
 	CString s;
-	int i;
+	int i = 0;
 
 	if ( len <= 0 )
 	{
@@ -184,34 +194,111 @@ void CClient::DataAvailable( const char * buffer, int len )
 	// add traffic control
 	CSocket::m_Traffic.AddTraffic(ettCONTROLRX,len);
 
-	i = 0;
-
-	s = m_sBuffer + CString().Set(buffer,len);
-	
-	// search last '|'
-	i = s.FindRev('|');
-	i++;
-
-	if ( i > 0 )
+	if ( m_bZMode )
 	{
-		m_sBuffer = s.Mid(0,i);
-		HandleMessage( m_sBuffer );
-	}
-
-	if ( i == s.Length() )
-	{
-		m_sBuffer = "";
+		int inlen = len;
+		CString trailing;
+		
+		if ( m_pZLib == 0 )
+		{
+			m_pZLib = new CZLib();
+		}
+		
+		int outlen = 1024*100; // same as what CTransfer uses
+		CByteArray * out = new CByteArray(outlen);
+		
+		//printf( "before inlen=%d outlen=%d ", inlen, outlen );
+		const int zlibstatus = m_pZLib->InflateZBlock( buffer, &inlen, (char*)out->Data(), &outlen );
+		//printf( "after inlen=%d outlen=%d\n", inlen, outlen );
+		
+		if ( zlibstatus == -1 )
+		{
+			printf("CClient: ZPipe decompress error, returning to normal mode\n");
+			delete m_pZLib;
+			m_pZLib = 0;
+			m_bZMode = false;
+		}
+		else if ( zlibstatus == 0 )
+		{
+			//printf("ZPipe: finished decompressing data, leaving zmode\n");
+			delete m_pZLib;
+			m_pZLib = 0;
+			m_bZMode = false;
+			
+			/*
+			 * Check for uncompressed data after compressed data.
+			 * In 0.3.19 there was usually data after the compressed data
+			 * which got ignored, because it put the compressed data
+			 * into m_sBuffer and did not send it to zlib until
+			 * some more data was received.
+			 */
+			if ( inlen < len )
+			{
+				trailing.Set( buffer+inlen, len-inlen );
+			}
+		}
+		
+		if ( ((zlibstatus == 0) || (zlibstatus == 1)) && (outlen > 0) )
+		{
+			s = m_sBuffer + CString().Set((char*)out->Data(),outlen);
+			m_sBuffer.Empty();
+			
+			if ( trailing.NotEmpty() )
+			{
+				s += trailing;
+			}
+			
+			//printf("ZPipe result: '%s'\n",s.Data());
+		}
+		
+		delete out;
 	}
 	else
 	{
-		m_sBuffer = s.Mid( i, s.Length()-i );
+		s = m_sBuffer + CString().Set(buffer,len);
 		
-		// need to process zline data
-		if ( m_bZMode == TRUE )
+		/* contents of buffer have been used, empty it to make sure it does not get used again */
+		m_sBuffer.Empty();
+	}
+	
+	/* If ZPipe is enabled, we must search forwards for $ZOn|
+	 * in case there is a | within the compressed data.
+	 *
+	 * And since hub supports negotiation is useless, we had better
+	 * assume ZPipe is always enabled.
+	 */
+	i = s.Find("$ZOn|");
+	if ( i >= 0 )
+	{
+		/* Handle all messages up to but not including $ZOn| */
+		HandleMessage( s.Mid(0,i) );
+		
+		/* Handle $ZOn| and advance i */
+		m_bZMode = true;
+		i += 5;
+		
+		if ( i < s.Length() )
 		{
-			HandleMessage( m_sBuffer );
-			m_sBuffer = "";
+			/*
+			 * Handle the compressed data.
+			 */
+			DataAvailable( s.Data()+i, s.Length()-i );
 		}
+		
+		return;
+	}
+	
+	/* No ZPipe support or $ZOn| not found */
+	i = s.FindRev('|') + 1;
+	
+	if ( i > 0 )
+	{
+		HandleMessage( s.Mid(0,i) );
+	}
+	
+	if ( i < s.Length() )
+	{
+		m_sBuffer = s.Mid( i, s.Length()-i );
 	}
 }
 
@@ -228,22 +315,22 @@ void CClient::Notify()
 		return;
 
 	// check away mode
-	if ( (m_bHandshake == FALSE) && ((time(0)-m_timeMyinfoTimeout) >= 30) )
+	if ( (m_bHandshake == false) && ((time(0)-m_timeMyinfoTimeout) >= 30) )
 	{
 		if ( CConfig::Instance()->GetAwayMode() != m_MyInfo.m_eAwayMode)
 		{
 			m_MyInfo.m_eAwayMode = CConfig::Instance()->GetAwayMode();
-			m_bUpdateMyinfo = TRUE;
+			m_bUpdateMyinfo = true;
 		}
 	}
 
-	if ( (m_bHandshake == FALSE) && (m_bUpdateMyinfo == TRUE) && ((time(0)-m_timeMyinfoTimeout) >= 30) )
+	if ( (m_bHandshake == false) && m_bUpdateMyinfo && ((time(0)-m_timeMyinfoTimeout) >= 30) )
 	{
-		m_bUpdateMyinfo     = FALSE;
+		m_bUpdateMyinfo     = false;
 		m_timeMyinfoTimeout = time(0);
 
 		if ( m_bSendMyinfo )
-			SendMyInfo( m_MyInfo.m_sNick, m_MyInfo.m_sComment, m_MyInfo.m_sUserSpeed, m_MyInfo.m_eAwayMode, m_MyInfo.m_sEMail, CString().setNum(m_MyInfo.m_nShared) );
+			SendMyInfo( &m_MyInfo );
 	}
 
 	if ( m_eReconnectState == ersENABLED )
@@ -316,55 +403,17 @@ int CClient::HandleMessage( const CString & message )
 	int err;
 	int pointer;
 	eDCMessage type;
-	CObject * Object;
-	CMessageHandler MessageHandler;
+	CDCMessage * Object = 0;
 	CString sAnswer,s;
-	CEncrypt Encrypt;
-	bool foundZOn = FALSE;
 
-	if ( message == "" )
+	if ( message.IsEmpty() )
 	{
 		return 0;
 	}
 
 	pointer = 0;
 
-	if ( m_bZMode == TRUE )
-	{
-		m_zBuffer.Append( message.Data(), message.Length() );
-		
-		CByteArray * inflated = new CByteArray();
-		CZLib * pZlib = new CZLib();
-		
-		unsigned long size = 0;
-		
-		size = pZlib->OneShotInflate( (const char*) m_zBuffer.Data(), m_zBuffer.Size(), inflated );
-		
-		if ( size == 0 )
-		{
-			// not an error - may be waiting for more data
-			//printf("CClient::HandleMessage: Failed to inflate zline data!\n");
-		}
-		else
-		{
-			//printf("CClient::HandleMessage: Inflated %ld to %ld\n", m_zBuffer.Size(), size);
-			CString mes;
-			mes.Set((const char*) inflated->Data(), size);
-			//printf("Inflated=%s\n", mes.Data());
-			
-			m_bZMode = FALSE;
-			m_zBuffer.SetSize(0);
-			
-			HandleMessage(mes);
-		}
-		
-		delete inflated;
-		delete pZlib;
-		
-		return 0;
-	}
-
-	while( (type=MessageHandler.Parse(&message,pointer,Object=0)) != DC_MESSAGE_PARSE_ERROR )
+	while( (type=m_pMessageHandler->Parse(&message,pointer,&Object)) != DC_MESSAGE_PARSE_ERROR )
 	{
 		if ( !Object )
 		{
@@ -378,57 +427,51 @@ int CClient::HandleMessage( const CString & message )
 				CMessageLock * msg = (CMessageLock*)Object;
 
 				// only handle the first lock in handshake mode
-				if ( m_bHandshake == TRUE )
+				if ( m_bHandshake )
 				{
-					Encrypt.Encrypt(msg->m_sData,sAnswer);
-
-					if ( msg->m_bExtProtocol == TRUE )
+#if DCLIB_HAS_SSL == 1
+					if ( m_bSSLMode )
 					{
-						s = "";
-#ifdef HAVE_SSL
-						if ( CConfig::Instance() &&
-						     (CConfig::Instance()->GetTransferCert() != "") &&
-						     (CConfig::Instance()->GetTransferKey() != "") )
-						{
-							s += "SSL ";
-						}
+						SendSSLInfo();
+					}
 #endif
-						// HubTopic
-						s += "HubTopic ";
-						// No $Hellos
-						s += "NoHello ";
-						// No $GetINFO
-						s += "NoGetINFO ";
+					CEncrypt::Encrypt(msg->m_sData,sAnswer);
 
+					if ( msg->m_bExtProtocol )
+					{
+						/*
+						 * Assumes NoHello NoGetINFO are supported.
+						 * The hub may not inform us what is actually enabled in its supports.
+						 */
+						m_bExtProtocol = true;
+						
+						// HubTopic No $Hellos No $GetINFO UserCommand
+						/*
+						 * UserIP2 now enabled always, if disabled we just do not use
+						 * the value the hub gives us, but it gets stored in case
+						 * the setting is changed without reconnecting to the hub.
+						 *
+						 * Similarly TTHSearch is always enabled because CConfig::Instance()->GetDisableHashList()
+						 * only affects the hashing of new files. However NOTTH is added to the tag.
+						 */
+						s = "HubTopic NoHello NoGetINFO TTHSearch UserCommand UserIP2 ";
+						
 						if ( CConfig::Instance() != 0 )
 						{
-							if ( CConfig::Instance()->GetDisableHashList() == FALSE )
-							{
-								s += "TTHSearch ";
-							}
-							
-							if ( CConfig::Instance()->GetUserIP2Enabled() == TRUE )
-							{
-								s += "UserIP2 ";
-							}
-							
-							if ( CConfig::Instance()->GetZLineEnabled() == TRUE )
+							if ( CConfig::Instance()->GetZPipeEnabled() )
 							{
 								// this is what DC++ 0.699 sends
 								// it may change later after testing finishes
 								s += "ZPipe0 ";
 							}
 							
-							if ( CConfig::Instance()->GetCompressedTransfers() == TRUE )
+							if ( CConfig::Instance()->GetCompressedTransfers() )
 							{
 								s += "GetZBlock ";
 							}
 						}
 
-						if ( s != "" )
-						{
-							SendSupports(s);
-						}
+						SendSupports(s);
 						// quicklist ??? http://forum.dcstats.net/showthread.php?s=&threadid=802
 					}
 
@@ -474,30 +517,26 @@ int CClient::HandleMessage( const CString & message )
 			{
 				CMessageHello * msg = (CMessageHello*)Object;
 
-				if ( (msg->m_sNick == m_MyInfo.m_sNick) && (m_bHandshake == TRUE) )
+				if ( (msg->m_sNick == m_MyInfo.m_sNick) && m_bHandshake )
 				{
-					SendVersion( m_sVersion );
+					SendVersion();
 
 					// get nicklist
-					if ( m_bHandleUserList == TRUE )
+					if ( m_bHandleUserList )
 						RequestNickList();
 
 					if ( m_bSendMyinfo )
-						SendMyInfo( m_MyInfo.m_sNick, m_MyInfo.m_sComment, m_MyInfo.m_sUserSpeed, m_MyInfo.m_eAwayMode, m_MyInfo.m_sEMail, CString().setNum(m_MyInfo.m_nShared) );
+						SendMyInfo( &m_MyInfo );
 
-					m_bHandshake = FALSE;
+					m_bHandshake = false;
 
 					// update myinfo
 					if ( m_bHandleMyinfo && CConnectionManager::Instance() )
 						CConnectionManager::Instance()->SendMyInfoToConnectedServers();
 				}
-				else if ( m_bHandleUserList == TRUE )
+				else if ( m_bHandleUserList )
 				{
-					if ( (m_UserList.AppendUser(msg->m_sNick) == TRUE) && (!m_bHandshake) && (m_MessageSupports.m_bNoGetInfo == FALSE) )
-						SendGetInfo(msg->m_sNick,GetNick());
-#ifdef HAVE_SSL
-					m_pMessageSSL.JoinHub(this,msg->m_sNick);
-#endif
+					m_UserList.AppendUser(msg->m_sNick);
 				}
 
 				break;
@@ -507,9 +546,9 @@ int CClient::HandleMessage( const CString & message )
 			{
 				CMessageMyInfo * msg = (CMessageMyInfo*)Object;
 
-				if( m_bHandleUserList == TRUE )
+				if ( m_bHandleUserList )
 				{
-					if ( m_UserList.UpdateUser(msg) == FALSE )
+					if ( m_UserList.UpdateUser(msg) == false )
 					{
 						delete Object;
 						Object = 0;
@@ -523,10 +562,10 @@ int CClient::HandleMessage( const CString & message )
 			{
 				CMessageQuit * msg = (CMessageQuit*)Object;
 
-				if ( m_bHandleUserList == TRUE )
+				if ( m_bHandleUserList )
 				{
 					m_UserList.RemoveUser(msg->m_sNick);
-#ifdef HAVE_SSL
+#if DCLIB_HAS_SSL == 1
 					m_pMessageSSL.LeaveHub(this,msg->m_sNick);
 #endif
 				}
@@ -536,11 +575,9 @@ int CClient::HandleMessage( const CString & message )
 
 			case DC_MESSAGE_NICKLIST:
 			{
-				CString *nick;
-				
 				CMessageNickList * msg = (CMessageNickList*)Object;
 
-				if( m_bHandleUserList == TRUE )
+				if ( m_bHandleUserList )
 				{
 					// 0: update nicklist 1: requested nicklist
 					if ( m_nNickListHandler == 1 )
@@ -552,14 +589,12 @@ int CClient::HandleMessage( const CString & message )
 
 					m_UserList.InitUserList(msg);
 					
-					if (m_MessageSupports.m_bNoGetInfo == TRUE)
+					if ( m_bExtProtocol == false )
 					{
-						break;
-					}
-					
-					for(nick=0;(nick=msg->m_NickList.Next(nick))!=0;)
-					{
-						SendGetInfo(*nick,GetNick());
+						for ( CString * nick = 0; (nick=msg->m_NickList.Next(nick))!=0; )
+						{
+							SendGetInfo(*nick,GetNick());
+						}
 					}
 				}
 
@@ -568,71 +603,57 @@ int CClient::HandleMessage( const CString & message )
 
 			case DC_MESSAGE_OPLIST:
 			{
-				CMessageNickList * msg = (CMessageNickList*)Object;
+				CMessageOpList * msg = (CMessageOpList*)Object;
 
 				// operators allready in the userlist
-				if( m_bHandleUserList == TRUE )
+				if ( m_bHandleUserList )
 				{
 					// init operator list
 					m_UserList.InitOperatorList(msg);
-					
-					m_nOpListHandler = 0;
 				}
 				break;
 			}
 
-			case DC_MESSAGE_SEARCH:
+			case DC_MESSAGE_SEARCH_FILE:
 			{
-				bool search = TRUE;
-				CMessageSearch * msg = (CMessageSearch*)Object;
+				bool search = true;
+				CMessageSearchFile * msg = (CMessageSearchFile*)Object;
 
-				if ( m_bHandleSearch == TRUE )
+				if ( m_bHandleSearch )
 				{
 					// check for loopback search
 					if ( msg->m_bLocal )
 					{
 						if ( msg->m_sSource == GetNick() )
 						{
-							search = FALSE;
+							search = false;
 						}
 					}
 					else
 					{
 						// disable global search, compare search with own ip and port
-						if ( CConfig::Instance() &&
-						     (msg->m_nPort == CConfig::Instance()->GetUDPListenPort()) &&
-						     (msg->m_sSource == CConfig::Instance()->GetUDPHostString(FALSE)) )
+						// also check private address space settings and IP
+						if ( CConfig::Instance() )
 						{
-							search = FALSE;
+							if ( (msg->m_nPort == CConfig::Instance()->GetUDPListenPort()) &&
+							     (msg->m_sSource == GetExternalIP(false)) )
+							{
+								search = false;
+							}
+							else if ( ( CConfig::Instance()->GetCheckPrivateAddressSpace() && 
+							            CNetAddr::IsPrivateI4(msg->m_sSource.Data()) ) ||
+								  ( CConfig::Instance()->GetPrivateAddressSpaceOnly() &&
+								    (CNetAddr::IsPrivateI4(msg->m_sSource.Data()) == false)) )
+							{
+								DPRINTF("Ignoring search from %s due to private address space settings\n",msg->m_sSource.Data());
+								search = false;
+							}
 						}
 					}
 
-					if ( (search == TRUE) && (CQueryManager::Instance() != 0) )
+					if ( search && (CQueryManager::Instance() != 0) )
 					{
-						// there may be a lot of search requests
-						// avoid looking up the same IP address each time
-						// so search for IP in cache
-						CString * ipaddr = 0;
-						if ( m_IPcache.Get( GetIP(), (CObject*&)ipaddr ) == 0 )
-						{
-							// found in cache;
-							//printf("Found in cache=%s\n", ipaddr->Data() );
-						}
-						else
-						{
-							// not found in cache
-							ipaddr = new CString();
-							*ipaddr = GetHostByName(GetIP());
-							
-							// add to cache
-							m_IPcache.Add( GetIP(), ipaddr );
-							
-							//printf("Looked up=%s\n", ipaddr->Data() );
-						}
-						
-						CString ipwithport = *ipaddr + ":" + CString().setNum(GetPort());
-						
-						CQueryManager::Instance()->SearchQuery( GetHubName(), ipwithport, GetNick(), msg );
+						CQueryManager::Instance()->SearchQuery( GetHubName(), GetResolvedIP(), GetNick(), msg );
 					}
 				}
 
@@ -648,13 +669,8 @@ int CClient::HandleMessage( const CString & message )
 					msg->m_sHubName = GetHubName();
 				}
 				
-				if( m_bHandleUserList == TRUE )
-				{
-					UpdateUserSlots( msg );
-				}
-
 				if ( CSearchManager::Instance() )
-					if ( CSearchManager::Instance()->HandleSearch( msg ) == TRUE )
+					if ( CSearchManager::Instance()->HandleSearch( msg ) )
 						Object = 0;
 				break;
 			}
@@ -663,10 +679,10 @@ int CClient::HandleMessage( const CString & message )
 			{
 				CMessageConnectToMe * msg = (CMessageConnectToMe*) Object;
 
-				if ( m_bHandleTransfer == TRUE )
+				if ( m_bHandleTransfer )
 				{
 					if ( CDownloadManager::Instance() )
-						CDownloadManager::Instance()->DLM_AddTransferRequest( msg->m_sHost, msg->m_nPort, GetHubName(), GetHost() );
+						CDownloadManager::Instance()->DLM_AddTransferRequest( msg->m_sHost, msg->m_nPort, msg->m_bCrypto, GetHubName(), GetHost() );
 				}
 
 				break;
@@ -676,16 +692,22 @@ int CClient::HandleMessage( const CString & message )
 			{
 				CMessageRevConnectToMe * msg = (CMessageRevConnectToMe*) Object;
 
-				if ( CConfig::Instance() && (GetMode() == ecmACTIVE) && (m_bHandleTransfer == TRUE) )
+				if ( CConfig::Instance() && (GetMode() == ecmACTIVE) && m_bHandleTransfer )
 				{
-					CString s = CConfig::Instance()->GetTCPHostString();
+#if DCLIB_HAS_SSL == 1
+					CMessageMyInfo usermyinfo;
+					bool crypto = ( m_MyInfo.m_bTLSFlag && m_UserList.GetUserMyInfo(msg->m_sDstNick,&usermyinfo) && usermyinfo.m_bTLSFlag );
+#else
+					bool crypto = false;
+#endif
+					CString s = GetExternalIP( true, crypto );
 
-					if ( s != "" )
+					if ( s.NotEmpty() )
 					{
 						if ( CDownloadManager::Instance() &&
-						     (CDownloadManager::Instance()->DLM_AddTransferRequest( msg->m_sDstNick, "", GetHubName(), GetHost() ) == TRUE) )
+						     CDownloadManager::Instance()->DLM_AddTransferRequest( msg->m_sDstNick, CString(), GetHubName(), GetHost() ) )
 						{
-							SendConnectToMe(msg->m_sDstNick,s);
+							SendConnectToMe(msg->m_sDstNick,s,crypto);
 						}
 					}
 				}
@@ -698,7 +720,7 @@ int CClient::HandleMessage( const CString & message )
 				CMessageForceMove * msg = (CMessageForceMove*) Object;
 
 				if ( CConfig::Instance() &&
-				     (CConfig::Instance()->GetForceMoveEnabled() == TRUE) && (m_bHandleForceMove == TRUE) )
+				     CConfig::Instance()->GetForceMoveEnabled() && m_bHandleForceMove )
 				{
 					SetHubName(msg->m_sHost);
 
@@ -714,14 +736,16 @@ int CClient::HandleMessage( const CString & message )
 			{
 				CMessagePrivateChat * msg = (CMessagePrivateChat*) Object;
 
+				/* default to receiving all messages if no CConfig */
 				if ( CConfig::Instance() &&
-				     (CConfig::Instance()->GetChatRecvOfflineMessages() == TRUE) && (m_UserList.IsUserOnline(msg->m_sSrcNick) == FALSE) )
+				     (CConfig::Instance()->GetChatRecvFromOffline() == false) &&
+				     (m_UserList.IsUserOnline(msg->m_sSrcNick) == false) )
 				{
 					delete Object;
 					Object = 0;
 					msg = 0;
 				}
-#ifdef HAVE_SSL
+#if DCLIB_HAS_SSL == 1
 				if ( msg )
 				{
 					m_pMessageSSL.PrivateChat(this,msg);
@@ -730,15 +754,13 @@ int CClient::HandleMessage( const CString & message )
 				break;
 			}
 
-			case DC_MESSAGE_CAPABILITIES:
-			{
-				m_MessageCapabilities = *((CMessageCapabilities*) Object);
-				break;
-			}
-
 			case DC_MESSAGE_SUPPORTS:
 			{
-				m_MessageSupports = *((CMessageSupports*) Object);
+				/*
+				 * Interesting. Since hubs may not send all the strings they support
+				 * it turns out there is currently nothing we can do with the supports message.
+				 */
+				
 				break;
 			}
 			
@@ -746,32 +768,32 @@ int CClient::HandleMessage( const CString & message )
 			{
 				CMessageUserIP * msg = (CMessageUserIP*) Object;
 				
-				if ( (CConfig::Instance() != 0) && (CConfig::Instance()->GetUserIP2Enabled() == TRUE) )
+				std::list<CString>::const_iterator nick_it = msg->m_lNicks.begin();
+				std::list<CString>::const_iterator ip_it = msg->m_lIPs.begin();
+				
+				while ( (nick_it != msg->m_lNicks.end()) && (ip_it != msg->m_lIPs.end()) )
 				{
-					if ( (GetMode() == ecmACTIVE) && (GetNick() == msg->m_sNick) )
+					if ( m_bHandleUserList )
 					{
-						// if using external IP and not network interface
-						if ( CConfig::Instance()->GetExternalIP() == TRUE )
+						m_UserList.SetUserIP( *nick_it, *ip_it );
+					}
+					
+					if ( GetNick() == *nick_it )
+					{
+						if ( CNetAddr::IsValidI4( ip_it->Data() ) )
 						{
-							CConfig::Instance()->SetHost(msg->m_sIP);
+							m_sExternalIP = *ip_it;
+							m_bGotIP = true;
 						}
 					}
+					
+					++nick_it;
+					++ip_it;
 				}
 				
 				break;
 			}
 			
-			case DC_MESSAGE_ZLINE:
-			{
-				foundZOn = TRUE;
-				
-				// nothing else needs to handle the object
-				delete Object;
-				Object = 0;
-				
-				break;
-			}
-			
 			default:
 			{
 				break;
@@ -780,8 +802,6 @@ int CClient::HandleMessage( const CString & message )
 
 		if (Object)
 		{
-			((CDCMessage*)Object)->m_eType = type;
-
 			if ( m_pCallback != 0 )
 			{
 				err = m_pCallback->notify( this, Object );
@@ -796,59 +816,21 @@ int CClient::HandleMessage( const CString & message )
 				delete Object;
 			}
 		}
-		
-		if ( foundZOn == TRUE )
-		{
-			m_bZMode = TRUE;
-			HandleMessage(message.Mid(pointer, message.Length() - pointer));
-			
-			break;
-		}
 	}
 
 	return 0;
 }
 
-/** */
-void CClient::UpdateUserSlots( CMessageSearchResult * pSearchResult )
-{
-	int err;
-	CMessageMyInfo myinfo,*mi;
-	
-	if ( m_UserList.UpdateUserSlots(pSearchResult) == TRUE )
-	{
-		if ( m_UserList.GetUserMyInfo( pSearchResult->m_sNick, &myinfo ) == TRUE )
-		{
-			mi = new CMessageMyInfo();
-			*mi = myinfo;
-
-			if ( m_pCallback != 0 )
-			{
-				err = m_pCallback->notify( this, mi );
-			}
-			else
-			{
-				err = DC_CallBack( mi );
-			}
-
-			if ( err == -1 )
-			{
-				delete mi;
-			}
-		}
-	}
-}
-
 /** */
 bool CClient::SetUserTransferInfo( CString nick, CDCMessage * msg )
 {
 	int err;
-	bool res = FALSE;
+	bool res = false;
 	CMessageMyInfo myinfo,*mi;
 
-	if ( m_UserList.SetUserTransferInfo(nick,msg) == TRUE )
+	if ( m_UserList.SetUserTransferInfo(nick,msg) )
 	{
-		if ( m_UserList.GetUserMyInfo( nick, &myinfo ) == TRUE )
+		if ( m_UserList.GetUserMyInfo( nick, &myinfo ) )
 		{
 			mi = new CMessageMyInfo();
 			*mi = myinfo;
@@ -867,7 +849,7 @@ bool CClient::SetUserTransferInfo( CString nick, CDCMessage * msg )
 				delete mi;
 			}
 			
-			res = TRUE;
+			res = true;
 		}
 	}
 	
@@ -880,13 +862,13 @@ int CClient::SendPrivateMessage( CString sNick, CString sTo, CString sMsg, CStri
 	int i=-1;
 	CString s;
 
-	if ( m_UserList.IsUserOnline( sTo ) || 
-	     (CConfig::Instance() && CConfig::Instance()->GetChatSendOfflineMessages()) )
+	if ( (CConfig::Instance() && CConfig::Instance()->GetChatSendToOffline()) ||
+	     m_UserList.IsUserOnline( sTo ) )
 	{
-#ifdef HAVE_SSL
+#if DCLIB_HAS_SSL == 1
 		s = m_pMessageSSL.EncryptMessage( this, sTo, sMsg );
 
-		if ( s != "" )
+		if ( s.NotEmpty() )
 		{
 			sMsg = s;
 		}
@@ -908,9 +890,11 @@ int CClient::RequestNickList()
 		res = CDCProto::RequestNickList();
 		
 		// if set, we will never recv a nicklist
-		if ( m_MessageSupports.m_bNoGetInfo == FALSE )
+		// hubs may not put NoGetINFO into their supports even if it is enabled
+		if ( m_bExtProtocol == false )
+		{
 			m_nNickListHandler = 1;
-		m_nOpListHandler = 1;
+		}
 	}
 	else
 	{
@@ -925,7 +909,127 @@ int CClient::Disconnect( bool force )
 {
 	UpdateReconnect( ersDISABLED );
 	
-	SetUsedPassword(FALSE);
+	SetUsedPassword(false);
 
 	return CConnection::Disconnect(force);
 }
+
+/** */
+CString CClient::GetExternalIP( bool addport, bool crypto ) const
+{
+	/*
+	 * If the hub does not support UserIP2 then the value we got from CConfig
+	 * will be used.
+	 */
+	if ( CConfig::Instance() && (CConfig::Instance()->GetUserIP2Enabled() == false) )
+	{
+		return CConfig::Instance()->GetTCPHostString( addport, crypto );
+	}
+	else if ( m_bGotIP )
+	{
+		if ( addport )
+		{
+			unsigned int port = 0;
+			
+			if ( crypto )
+			{
+				if ( CCryptoListenManager::Instance() )
+				{
+					port = CCryptoListenManager::Instance()->GetListenPort();
+				}
+			}
+			else
+			{
+				if ( CListenManager::Instance() )
+				{
+					port = CListenManager::Instance()->GetListenPort();
+				}
+			}
+			
+			if ( port != 0 )
+			{
+				return m_sExternalIP + ":" + CString::number(port);
+			}
+			else
+			{
+				return m_sExternalIP;
+			}
+		}
+		else
+		{
+			return m_sExternalIP;
+		}
+	}
+	else if ( CConfig::Instance() )
+	{
+		return CConfig::Instance()->GetTCPHostString( addport, crypto );
+	}
+	else
+	{
+		/* no IP from hub and no CConfig to get one from! */
+		return "127.0.0.1";
+	}
+}
+
+/** */
+int CClient::SendSearch( CMessageSearchFile * msg )
+{
+	
+	if ( msg->m_bLocal )
+	{
+		/* fix nick for passive search */
+		msg->m_sSource = GetNick();
+	}
+	else
+	{
+		/*
+		 * GetExternalIP() handles if we have an IP from the hub and if we are supposed to use it or not
+		 * it uses GetTCPHostString() but without adding the port this is the same as GetUDPHostString()
+		 */
+		msg->m_sSource = GetExternalIP(false,false) + ":" + CString::number(CConfig::Instance()->GetUDPListenPort());
+	}
+	
+	return CDCProto::SendSearch( msg );
+}
+
+/** */
+void CClient::SendSSLInfo()
+{
+	CMessageLog * log1 = new CMessageLog();
+	
+	log1->sMessage  = GetSSLVersion();
+	log1->sMessage += " connection using ";
+	log1->sMessage += GetSSLCipher();
+	
+	int err;
+	if ( m_pCallback )
+	{
+		err = m_pCallback->notify( this, log1 );
+	}
+	else
+	{
+		err = DC_CallBack( log1 );
+	}
+	
+	if ( err == -1 )
+	{
+		delete log1;
+	}
+	
+	CMessageLog * log2 = new CMessageLog();
+	log2->sMessage = VerifyPeerCertificate();
+	
+	if ( m_pCallback )
+	{
+		err = m_pCallback->notify( this, log2 );
+	}
+	else
+	{
+		err = DC_CallBack( log2 );
+	}
+	
+	if ( err == -1 )
+	{
+		delete log2;
+	}
+}
