@@ -2,7 +2,7 @@
                           cdcproto.h  -  description
                              -------------------
     begin                : Sun Dec 15 2002
-    copyright            : (C) 2002-2003 by Mathias Kster
+    copyright            : (C) 2002-2003 by Mathias Küster
     email                : mathen@users.berlios.de
  ***************************************************************************/
 
@@ -19,51 +19,59 @@
 #define CDCPROTO_H
 
 /**
-  *@author Mathias Kster
+  *@author Mathias Küster
+  *
+  * This builds various NMDC protocol strings and sends them.
+  *
+  * There are various legacy commands here, worst is
+  * dclib's own chunked Get, also GetZBlock and UGetBlock are no
+  * longer supported, ADCGET by filename is no longer
+  * supported, and the escaping for the filename with that is not
+  * the same as the current ADC protocol.
   */
 
 #include <dclib/dcos.h>
 #include <dclib/core/cconnection.h>
-#include <dclib/core/cthread.h>
+#include <dclib/core/cmutex.h>
 #include <dclib/core/ciconv.h>
+#include <dclib/core/types.h>
 
 class CMessageSearchFile;
+class CMessageMyInfo;
 
-class DLL_EXPORT CDCProto : public CConnection {
+class CDCProto : public CConnection {
 public: 
 	/** */
-	CDCProto();
+	CDCProto( CString remoteEncoding = CString() );
 	/** */
 	virtual ~CDCProto();
 
 	/** chat message */
 	int SendChat( CString sNick, CString s );
 	/** send version info */
-	int SendVersion( CString sVersion );
+	int SendVersion();
 	/** send myinfo */
-	int SendMyInfo( CString sNick, CString sComment, CString sConnectionType, eUserAwayMode eAwayMode, CString sEMail, CString sShare );
+	int SendMyInfo( CMessageMyInfo * myinfo );
 	/** send key */
 	int SendKey( CString s );
 	/** send validate nick */
 	int SendValidateNick( CString sNick );
-	/** send lock */
-	int SendLock();
 	/** send send */
 	int SendSend();
-	/** send my nick */
-	int SendMyNick( CString sNick );
+	/** send my nick and lock */
+	int SendMyNick( CString sNick, const CString & ref );
 	/** send get */
 	int SendGet( CString file, ulonglong pos, ulonglong size = 0 );
 	/** send GetZBlock */
-	int SendGetZBlock( CString file, ulonglong pos, ulonglong size = -1 );
+	int SendGetZBlock( CString file, ulonglong pos, ulonglong size = (ulonglong)-1 );
 	/** send UGetBlock */
-	int SendUGetBlock( CString file, ulonglong pos, ulonglong size = -1 );
+	int SendUGetBlock( CString file, ulonglong pos, ulonglong size = (ulonglong)-1 );
 	/** send UGetZBlock */
-	int SendUGetZBlock( CString file, ulonglong pos, ulonglong size = -1 );
+	int SendUGetZBlock( CString file, ulonglong pos, ulonglong size = (ulonglong)-1 );
 	/** send direction */
 	int SendDirection( eDirection Direction, int level );
 	/** send private message */
-	virtual int SendPrivateMessage( CString sNick, CString sTo, CString sMsg, CString sFromNick = "" );
+	virtual int SendPrivateMessage( CString sNick, CString sTo, CString sMsg, CString sFromNick = CString() );
 	/** send request nick list */
 	virtual int RequestNickList();
 	/** send rev connect to me */
@@ -77,11 +85,11 @@ public:
 	/** */
 	int SendGetListLen();
 	/** */
-	int SendConnectToMe( CString sDstNick, CString host );
+	int SendConnectToMe( CString sDstNick, CString host, bool encrypt = false );
 	/** */
 	int SendCanceled();
 	/** send string */
-	int SendString( CString message, bool encode = TRUE );
+	int SendString( CString message, bool encode = true );
 	/** send password */
 	int SendPass( CString pass );
 	/** operator kick */
@@ -99,15 +107,17 @@ public:
 	/** $Sending numBytes| */
 	int SendSending( ulonglong length );
 	/** */
-	int SendADCGet( CString type, CString tth, ulonglong pos, long long size, bool zlib = false );
+	int SendADCGet( eADCType type, CString tth, ulonglong pos, long long size, bool zlib = false, CString file = CString() );
 	/** */
-	int SendADCSnd( CString type, CString tth, ulonglong pos, long long size, bool zlib = false );
+	int SendADCSnd( eADCType type, CString tth, ulonglong pos, long long size, bool zlib = false, CString file = CString() );
 
 private:
 	/** */
-	CThread DCProtoThread;
-	/** CIconv object for doing text encoding */
-	CIconv * m_pIconv;
+	CMutex DCProtoMutex;
+	/** CIconv object for doing text encoding from local to remote */
+	CIconv * m_pLocalToRemote;
+	/** CIconv object for doing local to UTF-8 for certain commands */
+	CIconv * m_pLocalToUTF8;
 };
 
 #endif
