@@ -2,7 +2,7 @@
                           cbase64.cpp  -  description
                              -------------------
     begin                : Tue Jul 09 2002
-    copyright            : (C) 2002-2005 by Mathias Kster
+    copyright            : (C) 2002-2005 by Mathias Küster
     email                : mathen@users.berlios.de
  ***************************************************************************/
 
@@ -15,16 +15,16 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "cbase64.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <dclib/core/cbytearray.h>
-
-#include "cbase64.h"
+#include "cbytearray.h"
 
 // original from mutt
 
-static int Index_64[128] = {
+const signed char CBase64::Index_64[128] = {
 -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
 -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
 -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,62, 63,-1,-1,-1,
@@ -35,7 +35,7 @@ static int Index_64[128] = {
 41,42,43,44, 45,46,47,48, 49,50,51,-1, -1,-1,-1,-1,
 };
 
-static char B64Chars[64] = {
+const char CBase64::B64Chars[64] = {
 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd',
 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
@@ -44,7 +44,7 @@ static char B64Chars[64] = {
 };
 
 #define base64val(c) Index_64[(unsigned int)(c)]
-#define base64chars(c) B64Chars[(unsigned int)(c&0x3F)]
+#define base64chars(c) B64Chars[(unsigned int)((c)&0x3F)]
 
 /** */
 CBase64::CBase64()
@@ -56,15 +56,13 @@ CBase64::~CBase64()
 {
 }
 
-#define BAD     -1
-
 /** */
 CString CBase64::Encode( CString string )
 {
 	CString outs;
 	CByteArray in,out;
 
-	in.Append(string.Data(),string.Length()+1);
+	in.Append(string.Data(),string.Length());
 
 	Encode(&out,&in);
 
@@ -121,7 +119,7 @@ void CBase64::Encode(  CByteArray * dst, CByteArray * src )
 }
 
 /** */
-int CBase64::Decode( CByteArray * dst, CString * string )
+unsigned long CBase64::Decode( CByteArray * dst, CString * string )
 {
 	CByteArray in;
 
@@ -132,7 +130,7 @@ int CBase64::Decode( CByteArray * dst, CString * string )
 
 /* Convert '\0'-terminated base 64 string to raw bytes.
  * Returns length of returned buffer, or -1 on error */
-int CBase64::Decode( CByteArray * dst, CByteArray * src )
+unsigned long CBase64::Decode( CByteArray * dst, CByteArray * src )
 {
 	unsigned char *current,*result;
 	unsigned char c;
