@@ -15,6 +15,8 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "ciconv.h"
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -27,28 +29,25 @@
 #include <stdlib.h>
 #include <errno.h>
 
-#include "ciconv.h"
-
 /** calls iconv_open */
-CIconv::CIconv( CString from, CString to )
+CIconv::CIconv( const CString & from, const CString & to )
 {
-	if ( (from.ToLower() == "none") || (to.ToLower() == "none") )
+	if ( from == to )
 	{
-		// printf( "CIconv: encoding is disabled\n" );
-		disabled = TRUE;
+		disabled = true;
 	}
 	else
 	{
 		cd = iconv_open( to.Data(), from.Data() );
 		if ( cd == (iconv_t)(-1) )
 		{
-			perror( "CIconv::CIconv" );
+			perror( "CIconv iconv_open" );
 			printf( "CIconv: encoding disabled - iconv_open failed\n" );
-			disabled = TRUE;
+			disabled = true;
 		}
 		else
 		{
-			disabled = FALSE;
+			disabled = false;
 		}
 	}
 }
@@ -56,33 +55,40 @@ CIconv::CIconv( CString from, CString to )
 /** calls iconv_close */
 CIconv::~CIconv()
 {
-	if ( disabled == FALSE )
+	if ( disabled == false )
 	{
 		iconv_close( cd );
 	}
 }
 
 /** calls iconv */
-CString CIconv::encode( CString orig )
+CString CIconv::encode( const char * orig, const size_t origlen )
 {
-	if ( disabled == TRUE )
+	if ( orig == 0 || origlen == 0 )
 	{
-		return orig;
+		return CString();
 	}
 	
-	if ( orig == "" )
-	{
-		return "";
-	}
-	
-	CString result = "";
+	CString result;
 	size_t retval;
 	
-	size_t inleft = orig.Length();
-	size_t outleft = inleft * 4;
+	size_t inleft = origlen;
+	
+	size_t outleft;
+	if ( inleft < 1024 )
+	{
+		outleft = inleft * 4;
+	}
+	else
+	{
+		outleft = 4096;
+	}
+	
+	const char * inbuf = orig;
+	
+	// allocate an extra byte to ensure the data is \0 terminated
+	char * outbuf = (char*) calloc( 1, outleft + 1 );
 	
-	char * inbuf = orig.Data();
-	char * outbuf = (char*) calloc( 1, outleft );
 	char * outstart = outbuf;
 	
 	while ( inleft > 0 )
@@ -102,6 +108,17 @@ CString CIconv::encode( CString orig )
 				outbuf++;
 				outleft--;
 			}
+			else if ( errno == E2BIG ) // not enough room in output buffer for next char
+			{
+				// CString += uses strlen to compute the length
+				// which will be correct since we zero filled the memory
+				// with calloc, so we don't need to compute any sizes here
+				result += outstart;
+				free(outstart);
+				outleft = 4096;
+				outbuf = (char*) calloc( 1, outleft + 1 );
+				outstart = outbuf;
+			}
 			else
 			{
 				printf("iconv fail: %d\n", retval);
@@ -113,7 +130,7 @@ CString CIconv::encode( CString orig )
 		}
 	}
 		
-	result = outstart;
+	result += outstart;
 	free(outstart);
 	
 	//printf( "orig=\"%s\" len=%d\nresult=\"%s\" len=%d\n", orig.Data(), orig.Length(), result.Data(), result.Length() );
