@@ -2,7 +2,7 @@
                           csocket.cpp  -  description
                              -------------------
     begin                : Sat Oct 6 2001
-    copyright            : (C) 2001-2003 by Mathias Kster
+    copyright            : (C) 2001-2003 by Mathias Küster
     email                : mathen@users.berlios.de
  ***************************************************************************/
 
@@ -15,6 +15,8 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "csocket.h"
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -24,11 +26,14 @@
 #include <errno.h>
 #include <signal.h>
 
-#ifndef WIN32
+#ifdef WIN32
+#include <winsock2.h>
+#else
 #include <netdb.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <sys/time.h>
+#include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <netinet/in.h>
@@ -54,24 +59,11 @@
 #define MSG_NOSIGNAL      0
 #endif
 
-#include <dclib/dcos.h>
-#include <dclib/core/casyncdns.h>
-#include <dclib/core/clogfile.h>
-#include <dclib/core/cssl.h>
-
-#ifdef WIN32
-//#include <winsock2.h>
-//#include <ws2tcpip.h>
-#include <iprtrmib.h>
-#include <iphlpapi.h>
-#endif
-
-// Needed on OSF/1 
-#ifndef hstrerror
-#define hstrerror(x) ((const char*)"Resolver error.")
-#endif
-
-#include "csocket.h"
+#include "../dcos.h"
+#include "casyncdns.h"
+#include "clogfile.h"
+#include "cssl.h"
+#include "cnetaddr.h"
 
 #ifdef HAVE_SOCKS
 extern "C" {
@@ -81,66 +73,36 @@ extern "C" {
 
 // init static socket vars
 CTraffic CSocket::m_Traffic = CTraffic();
-CThread CSocket::m_cs_ghbn = CThread();
 eSocketLog CSocket::m_eSocketLog = eslNONE;
 
 CSocket::CSocket( eSocketType type )
 {
 	SocketType    = type;
 	iHandle       = INVALID_SOCKET;
-	sError        = "";
 	m_eSocketMode = esmSOCKET;
-	m_bSSLInit    = FALSE;
-#ifdef HAVE_SSL
+
 	m_pCTX = 0;
 	m_pSSL = 0;
-#endif
-
-#ifdef WIN32
-	WORD wVersionRequested;
-	WSADATA wsaData;
-	int err;
- 
-	wVersionRequested = MAKEWORD( 2, 2 );
- 
-	err = WSAStartup( wVersionRequested, &wsaData );
-
-	if ( err != 0 )
-	{
-		/* Tell the user that we could not find a usable */
-		/* WinSock DLL.                                  */
-		return;
-	}
- 
-	/* Confirm that the WinSock DLL supports 2.2.*/
-	/* Note that if the DLL supports versions greater    */
-	/* than 2.2 in addition to 2.2, it will still return */
-	/* 2.2 in wVersion since that is the version we      */
-	/* requested.                                        */
- 
-	if ( LOBYTE( wsaData.wVersion ) != 2 ||
-		HIBYTE( wsaData.wVersion ) != 2 )
-	{
-		/* Tell the user that we could not find a usable */
-		/* WinSock DLL.                                  */
-		WSACleanup( );
-		return; 
-	}
- 
-	/* The WinSock DLL is acceptable. Proceed. */
-#endif
 }
 
 CSocket::~CSocket()
 {
-	if ( iHandle != INVALID_SOCKET )
+	Disconnect();
+
+#if DCLIB_USES_OPENSSL == 1
+	/* This is in case iHandle == INVALID_SOCKET but somehow m_pSSL / m_pCTX are still here */
+	if ( m_pSSL )
 	{
-#ifdef WIN32
-		closesocket(iHandle);
-#else
-		close(iHandle);
-#endif
+		SSL_free(m_pSSL);
+		m_pSSL = 0;
 	}
+	
+	if ( m_pCTX )
+	{
+		SSL_CTX_free(m_pCTX);
+		m_pCTX = 0;
+	}
+#endif
 }
 
 /** */
@@ -159,788 +121,30 @@ int CSocket::SetSocket( int handle, eSocketType sockettype )
 	SocketType = sockettype;
 	iHandle    = handle;
 
+#if DCLIB_USES_OPENSSL == 1
+	if ( (m_eSocketMode == esmFULLSSLSERVER) || (m_eSocketMode == esmFULLSSLCLIENT) )
+	{
+		if ( SSL_set_fd(m_pSSL,iHandle) == 0 )
+		{
+			sError  = "CSocket::SetSocket: SSL_set_fd failed: ";
+			sError += ERR_reason_error_string( ERR_get_error() );
+			return -1;
+		}
+	}
+#endif
+
 	return 0;
 }
 
-/** rfc1918 */
-bool CSocket::IsPrivateAddressSpace( const char * cp )
-{
-	unsigned long int i;
-	 
-	// sanity check
-	if ( !cp )
-		return FALSE;
-	
-	i = inet_addr(cp);
-
-	if ( i == (unsigned long int)-1 )
-		return FALSE;
-
-	/** 10.0.0.0 - 10.255.255.255
-	  * 172.16.0.0 - 172.31.255.255
-	  * 192.168.0.0 - 192.168.255.255
-	*/
-	if ( ((i | inet_addr("10.255.255.255"))  == inet_addr("10.255.255.255")) ||
-	     ((i | inet_addr("172.16.255.255"))  == inet_addr("172.31.255.255")) ||
-	     ((i | inet_addr("192.168.255.255")) == inet_addr("192.168.255.255")) )
-		return TRUE;
-
-	return FALSE;
-}
-
-/** */
-bool CSocket::GetHostByName( const char * name, struct sockaddr_in * sin )
-{
-	m_cs_ghbn.Lock();
-
-	bool err = FALSE;
-	struct hostent * hp;
-
-	// sanity check
-	if ( name && sin )
-	{
-		memset(sin,0,sizeof(struct sockaddr_in));
-
-		if ( (hp = gethostbyname(name)) != 0 )
-		{
-			if ( hp->h_addr != 0 )
-			{
-		 		memcpy((void *)&sin->sin_addr, hp->h_addr, sizeof(sin->sin_addr) ); //hp->h_length);
-				err = TRUE;
-			}
-		}
-	}
-
-	m_cs_ghbn.UnLock();
-
-	return err;
-}
-
-/** */
-CString CSocket::GetHostByName( CString Host )
-{
-	CString s = "";
-	struct sockaddr_in sin;
-
-	if ( Host != "" )
-	{
-		if ( GetHostByName( Host.Data(), &sin ) == TRUE )
-		{
-			s = inet_ntoa(sin.sin_addr);
-		}
-	}
-
-	return s;
-}
-
-/** */
-void CSocket::ParseHost( CString host, CString & ip, unsigned int & port )
-{
-	int i;
-
-	ip = "";
-
-	// remove all spaces
-	host = host.Replace(" ","");
-
-	i = host.Find(':');
-
-	if ( i >= 0 )
-	{
-		ip = host.Mid(i+1,host.Length()-i-1);
-	}
-
-	port = 0;
-
-	if ( (i < 0) || (ip == "") )
-	{
-		ip   = host;
-	}
-	else
-	{
-		if ( ip.asINT() < 0 )
-			port += 65536;
-		port += ip.asINT();
-		ip   = host.Mid(0,i);
-	}	
-}
-
-/** */
-int CSocket::GetInterfaceList( CList<CString> * interfacelist )
-{
-	if ( !interfacelist )
-	{
-		return -1;
-	}
-
-#ifdef WIN32
-	// get the interfaces from win32
-	LONG Status;
-	ULONG Result;
-	HKEY AdapKey,OneAdapKey;
-	TCHAR AdapName[256];
-	DWORD dim;
-	UCHAR buffer[256];
-	CString name,instance;
-	int i;
-
-	// ip-helper
-	ULONG buf_size;
-	DWORD getiftable_rc;
-	char *buff = NULL;
-	MIB_IFTABLE *ift;
-	DWORD count;
-	MIB_IFROW *ifr;
-	IP_ADAPTER_INFO *ai;
-
-	// iphelper interface table
-	buf_size = 0;
-	getiftable_rc = GetIfTable((PMIB_IFTABLE) buff, &buf_size, TRUE);
-
-	if (getiftable_rc == ERROR_INSUFFICIENT_BUFFER)
-	{
-		buff = (char *) malloc(buf_size);
-		memset(buff, 0, buf_size);
-
-		getiftable_rc = GetIfTable((PMIB_IFTABLE) buff, &buf_size, TRUE);
-
-		if (getiftable_rc == NO_ERROR)
-		{
-			ift = (MIB_IFTABLE *) buff;
-			count = ift->dwNumEntries;
-
-			for (i=0;i<count;i++)
-			{
-				ifr = (ift->table) + i;
-
-				if ( ifr->dwDescrLen > 0 )
-				{
-					CString * s=0;
-					while( (s=interfacelist->Next(s)) != 0 )
-						if ( *s == (const char*)ifr->bDescr )
-							break;
-					if ( !s )
-						interfacelist->Add( new CString((const char*)ifr->bDescr) );
-				}
-			}
-		}
-
-		free(buff);
-	}
-
-	// iphelper adapter info
-	buf_size = 0;
-	getiftable_rc = GetAdaptersInfo((PIP_ADAPTER_INFO) buff, &buf_size );
-
-	if (getiftable_rc == ERROR_BUFFER_OVERFLOW)
-	{
-		buff = (char *) malloc(buf_size);
-		memset(buff, 0, buf_size);
-
-		getiftable_rc = GetAdaptersInfo((PIP_ADAPTER_INFO) buff, &buf_size );
-
-		if (getiftable_rc == ERROR_SUCCESS)
-		{
-			ai = (IP_ADAPTER_INFO *) buff;
-
-			for (;ai;ai=ai->Next)
-			{
-				CString * s=0;
-				while( (s=interfacelist->Next(s)) != 0 )
-					if ( *s == (const char*)ai->Description )
-						break;
-				if ( !s )
-					interfacelist->Add( new CString((const char*)ai->Description));
-			}
-		}
-
-		free(buff);
-	}
-
-	// win 95/98
-	Status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-		TEXT("SYSTEM\\CurrentControlSet\\Services\\Class\\Net"),
-		0,
-		KEY_READ,
-		&AdapKey);
-
-	if ( Status == ERROR_SUCCESS )
-	{
-		i=0;
-
-		// Get the size to allocate for the original device names
-		while( (Result=RegEnumKey(AdapKey,i,AdapName,sizeof(AdapName)/2)) == ERROR_SUCCESS )
-		{
-			Status = RegOpenKeyEx(AdapKey,AdapName,0,KEY_READ,&OneAdapKey);
-
-			if ( Status == ERROR_SUCCESS )
-			{
-				name     = "";
-				instance = (char*)AdapName;
-				dim = 256;
-
-				Status = RegQueryValueEx(OneAdapKey,"DriverDesc",NULL,NULL,buffer,&dim);
-
-				if ( Status == ERROR_SUCCESS )
-				{
-					name = (char*)buffer;
-				}
-
-				RegCloseKey(OneAdapKey);
-
-				if ( (instance != "") && (name != "") )
-				{
-					// now we check if the interface exist
-					instance  = "SYSTEM\\CurrentControlSet\\Services\\Class\\NetTrans\\"+instance;
-
-					Status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-							TEXT(instance.Data()),
-							0,
-							KEY_READ,
-							&OneAdapKey);
-
-					if ( Status == ERROR_SUCCESS )
-					{
-						CString * s=0;
-						while( (s=interfacelist->Next(s)) != 0 )
-							if ( *s == name )
-								break;
-						if ( !s )
-							interfacelist->Add( new CString(name) );
-						RegCloseKey(OneAdapKey);
-					}
-				}
-			}
-
-			i++;
-		}
-
-		RegCloseKey(AdapKey);
-	}
-
-	// win nt/2k
-	Status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-		TEXT("SYSTEM\\CurrentControlSet\\Control\\Class\\{4D36E972-E325-11CE-BFC1-08002BE10318}"),
-		0,
-		KEY_READ,
-		&AdapKey);
-
-	if ( Status == ERROR_SUCCESS )
-	{
-		i=0;
-
-		// Get the size to allocate for the original device names
-		while( (Result=RegEnumKey(AdapKey,i,AdapName,sizeof(AdapName)/2)) == ERROR_SUCCESS )
-		{
-			Status = RegOpenKeyEx(AdapKey,AdapName,0,KEY_READ,&OneAdapKey);
-
-			if ( Status == ERROR_SUCCESS )
-			{
-				name     = "";
-				instance = "";
-
-				dim  = 256;
-				Status = RegQueryValueEx(OneAdapKey,"NetCfgInstanceId",NULL,NULL,buffer,&dim);
-
-				if ( Status == ERROR_SUCCESS )
-				{
-					dim = 256;
-					instance = (char*)buffer;
-
-					Status = RegQueryValueEx(OneAdapKey,"DriverDesc",NULL,NULL,buffer,&dim);
-
-					if ( Status == ERROR_SUCCESS )
-					{
-						name = (char*)buffer;
-					}
-				}
-
-				RegCloseKey(OneAdapKey);
-
-				if ( (instance != "") && (name != "") )
-				{
-					// now we check if the interface exist
-					instance  = "SYSTEM\\CurrentControlSet\\Services\\"+instance +"\\Parameters\\Tcpip";
-
-					Status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-							TEXT(instance.Data()),
-							0,
-							KEY_READ,
-							&OneAdapKey);
-
-					if ( Status == ERROR_SUCCESS )
-					{
-						CString * s=0;
-						while( (s=interfacelist->Next(s)) != 0 )
-							if ( *s == name )
-								break;
-						if ( !s )
-							interfacelist->Add( new CString(name) );
-						RegCloseKey(OneAdapKey);
-					}
-				}
-			}
-
-			i++;
-		}
-
-		RegCloseKey(AdapKey);
-	}
-#else
-	int sock = -1;
-	struct ifreq ifreqa[256];
-	int i;
-	struct ifconf *pifconf;
-	struct ifreq * pifreq;
-
-	sock = socket( PF_INET, SOCK_DGRAM, 0 );
-
-	if ( sock != -1 )
-	{
-		pifconf = (struct ifconf*) malloc(sizeof(ifconf));
-
-		pifconf->ifc_len = 255*32;
-		pifconf->ifc_ifcu.ifcu_req = ifreqa;
-
-		i = ioctl( sock, SIOCGIFCONF, pifconf );
-
-		if ( i != -1 )
-		{
-			if ( pifconf->ifc_len > 0 )
-			{
-				for( pifreq = pifconf->ifc_ifcu.ifcu_req, i = 0;
-				     pifconf->ifc_len > i;)
-				{
-					CString * s=0;
-					while( (s=interfacelist->Next(s)) != 0 )
-						if ( *s == ((char*)pifreq->ifr_name) )
-							break;
-					if ( !s )
-						interfacelist->Add( new CString((char*)pifreq->ifr_name) );
-#ifdef HAVE_SA_LEN
-					i += IFNAMSIZ + pifreq->ifr_addr.sa_len;
-#else
-					i += sizeof( struct ifreq );
-#endif
-					pifreq = (struct ifreq *)((caddr_t)pifconf->ifc_ifcu.ifcu_req + i);
-				}
-			}
-		}
-
-		close(sock);
-		free(pifconf);
-	}
-#endif
-
-	return interfacelist->Count();
-}
-
-/** */
-CString CSocket::GetInterfaceIP( CString iface )
-{
-	CString s = "";
-#ifdef WIN32
-	// get the interfaces from win32
-	LONG Status;
-	ULONG Result;
-	HKEY AdapKey,OneAdapKey;
-	TCHAR AdapName[256];
-	DWORD dim;
-	DWORD RegType;
-	DWORD DHCPEnabled; 
-	UCHAR buffer[256];
-	CString name,instance;
-	int i;
-
-	// ip-helper
-	ULONG buf_size;
-	DWORD getiftable_rc;
-	char *buff = NULL;
-	MIB_IFTABLE *ift;
-	DWORD count;
-	DWORD index=0xFFFFFFFF;
-	MIB_IFROW *ifr;
-	IP_ADAPTER_INFO *ai;
-	MIB_IPADDRTABLE *ipt;
-	MIB_IPADDRROW *ipr;
-
-	// iphelper interface table
-	buf_size = 0;
-	buff = NULL;
-	getiftable_rc = GetIfTable((PMIB_IFTABLE) buff, &buf_size, TRUE);
-
-	if (getiftable_rc == ERROR_INSUFFICIENT_BUFFER)
-	{
-		buff = (char *) malloc(buf_size);
-		memset(buff, 0, buf_size);
-
-		getiftable_rc = GetIfTable((PMIB_IFTABLE) buff, &buf_size, TRUE);
-
-		if (getiftable_rc == NO_ERROR)
-		{
-			ift = (MIB_IFTABLE *) buff;
-			count = ift->dwNumEntries;
-
-			for (i=0;i<count;i++)
-			{
-				ifr = (ift->table) + i;
-
-				if ( iface == (const char*)ifr->bDescr )
-				{
-					index = ifr->dwIndex;
-					break;
-				}
-			}
-		}
-
-		free(buff);
-	}
-
-	// get ip from nettable
-	if ( index != 0xFFFFFFFF )
-	{
-		buf_size = 0;
-		buff = NULL;
-		getiftable_rc = GetIpAddrTable((PMIB_IPADDRTABLE) buff, &buf_size, TRUE );
-
-		if (getiftable_rc == ERROR_INSUFFICIENT_BUFFER)
-		{
-			buff = (char *) malloc(buf_size);
-			memset(buff, 0, buf_size);
-
-			getiftable_rc = GetIpAddrTable((PMIB_IPADDRTABLE) buff, &buf_size, TRUE );
-
-			if (getiftable_rc == NO_ERROR)
-			{
-				ipt = (MIB_IPADDRTABLE *) buff;
-				count = ipt->dwNumEntries;
-
-				for (i=0;i<count;i++)
-				{
-					ipr = (ipt->table) + i;
-
-					if ( ipr->dwIndex == index )
-					{
-						struct in_addr in;
-
-						in.s_addr = ipr->dwAddr;
-						s = inet_ntoa(in);
-						break;
-					}
-				}
-			}
-		}
-
-		free(buff);
-	}
-
-	if ( s != "" )
-	{
-		return s;
-	}
-
-	// iphelper adapter info
-	buf_size = 0;
-	buff = NULL;
-	getiftable_rc = GetAdaptersInfo((PIP_ADAPTER_INFO) buff, &buf_size );
-
-	if (getiftable_rc == ERROR_BUFFER_OVERFLOW)
-	{
-		buff = (char *) malloc(buf_size);
-		memset(buff, 0, buf_size);
-
-		getiftable_rc = GetAdaptersInfo((PIP_ADAPTER_INFO) buff, &buf_size );
-
-		if (getiftable_rc == ERROR_SUCCESS)
-		{
-			ai = (IP_ADAPTER_INFO *) buff;
-
-			for (;ai;ai=ai->Next)
-			{
-				if ( iface == ai->Description )
-				{
-					s = ai->IpAddressList.IpAddress.String;
-					break;
-				}
-			}
-		}
-
-		free(buff);
-	}
-
-	if ( s != "" )
-	{
-		return s;
-	}
-
-	// win 95/98
-	Status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-		TEXT("SYSTEM\\CurrentControlSet\\Services\\Class\\Net"),
-		0,
-		KEY_READ,
-		&AdapKey);
-
-	if ( Status == ERROR_SUCCESS )
-	{
-		i=0;
-
-		// Get the size to allocate for the original device names
-		while( (Result=RegEnumKey(AdapKey,i,AdapName,sizeof(AdapName)/2)) == ERROR_SUCCESS )
-		{
-			Status = RegOpenKeyEx(AdapKey,AdapName,0,KEY_READ,&OneAdapKey);
-
-			if ( Status == ERROR_SUCCESS )
-			{
-				name     = "";
-				instance = (char*)AdapName;
-				dim = 256;
-
-				Status = RegQueryValueEx(OneAdapKey,"DriverDesc",NULL,NULL,buffer,&dim);
-
-				if ( Status == ERROR_SUCCESS )
-				{
-					name = (char*)buffer;
-
-					if ( name != iface )
-					{
-						name = "";
-					}
-				}
-
-				RegCloseKey(OneAdapKey);
-
-				if ( (instance != "") && (name != "") )
-				{
-					// now we check if the interface exist
-					instance  = "SYSTEM\\CurrentControlSet\\Services\\Class\\NetTrans\\"+instance;
-
-					Status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-							TEXT(instance.Data()),
-							0,
-							KEY_READ,
-							&OneAdapKey);
-
-					if ( Status == ERROR_SUCCESS )
-					{
-						dim = 4;
-						Status = RegQueryValueEx(OneAdapKey,TEXT("EnableDHCP"),NULL,&RegType,(LPBYTE)&DHCPEnabled,&dim);
-
-						if ( Status != ERROR_SUCCESS )
-						{
-							DHCPEnabled = 0;
-						}
-
-						if ( DHCPEnabled == 0 )
-						{
-							dim = 265;
-							Status=RegQueryValueEx(OneAdapKey,"IPAddress",NULL,NULL,buffer,&dim);
-							if ( Status == ERROR_SUCCESS )
-							{
-								s = (char*)buffer;
-							}
-						}
-						else
-						{
-							dim = 256;
-							Status = RegQueryValueEx(OneAdapKey,"DhcpIPAddress",NULL,NULL,buffer,&dim);
-
-							if ( Status == ERROR_SUCCESS )
-							{
-								s = (char*)buffer;
-							}
-						}
-
-						RegCloseKey(OneAdapKey);
-
-						break;
-					}
-
-				}
-			}
-
-			i++;
-		}
-
-		RegCloseKey(AdapKey);
-	}
-
-	if ( s != "" )
-	{
-		return s;
-	}
-
-	// win nt/2k
-	Status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-		TEXT("SYSTEM\\CurrentControlSet\\Control\\Class\\{4D36E972-E325-11CE-BFC1-08002BE10318}"),
-		0,
-		KEY_READ,
-		&AdapKey);
-
-	if ( Status == ERROR_SUCCESS )
-	{
-		i=0;
-
-		// Get the size to allocate for the original device names
-		while( (Result=RegEnumKey(AdapKey,i,AdapName,sizeof(AdapName)/2)) == ERROR_SUCCESS )
-		{
-			Status = RegOpenKeyEx(AdapKey,AdapName,0,KEY_READ,&OneAdapKey);
-
-			if ( Status == ERROR_SUCCESS )
-			{
-				name     = "";
-				instance = "";
-
-				dim = 256;
-				Status = RegQueryValueEx(OneAdapKey,"DriverDesc",NULL,NULL,buffer,&dim);
-
-				if ( Status == ERROR_SUCCESS )
-				{
-					name = (char*)buffer;
-
-					if ( name == iface )
-					{
-						// found
-						dim = 256;
-						Status = RegQueryValueEx(OneAdapKey,"NetCfgInstanceId",NULL,NULL,buffer,&dim);					
-
-						if ( Status == ERROR_SUCCESS )
-						{
-							instance = (char*)buffer;
-						}
-					}
-				}
-
-				RegCloseKey(OneAdapKey);
-
-				if ( (instance != "") && (name != "") )
-				{
-					// now we get the ip
-					instance = "SYSTEM\\CurrentControlSet\\Services\\"+instance+"\\Parameters\\Tcpip";
-
-					Status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-							TEXT(instance.Data()),
-							0,
-							KEY_READ,
-							&OneAdapKey);
-
-					if ( Status == ERROR_SUCCESS )
-					{
-						dim = 4;
-						Status = RegQueryValueEx(OneAdapKey,TEXT("EnableDHCP"),NULL,&RegType,(LPBYTE)&DHCPEnabled,&dim);
-
-						if ( Status != ERROR_SUCCESS )
-						{
-							DHCPEnabled = 0;
-						}
-
-						if ( DHCPEnabled == 0 )
-						{
-							dim = 265;
-							Status=RegQueryValueEx(OneAdapKey,"IPAddress",NULL,NULL,buffer,&dim);
-							if ( Status == ERROR_SUCCESS )
-							{
-								s = (char*)buffer;
-							}
-						}
-						else
-						{
-							dim = 256;
-							Status = RegQueryValueEx(OneAdapKey,"DhcpIPAddress",NULL,NULL,buffer,&dim);
-
-							if ( Status == ERROR_SUCCESS )
-							{
-								s = (char*)buffer;
-							}
-						}
-
-						RegCloseKey(OneAdapKey);
-
-						break;
-					}
-				}
-			}
-
-			i++;
-		}
-
-		RegCloseKey(AdapKey);
-	}
-#else
-	int sock = -1;
-	struct ifreq ifreqa[256];
-	int i;
-	struct ifconf *pifconf;
-	struct ifreq * pifreq;
-	struct ifreq ifr;
-
-	sock = socket( PF_INET, SOCK_DGRAM, 0 );
-
-	if ( sock == -1 )
-	{
-		return s;
-	}
-
-	pifconf = (struct ifconf*) malloc(sizeof(ifconf));
-
-	pifconf->ifc_len = 255*32;
-	pifconf->ifc_ifcu.ifcu_req = ifreqa;
-
-	i = ioctl( sock, SIOCGIFCONF, pifconf );
-
-	if ( i == -1 )
-	{
-		close(sock);
-		free(pifconf);
-		return s;
-	}
-
-	if ( pifconf->ifc_len == 0 )
-	{
-		close(sock);
-		free(pifconf);
-		return s;
-	}
-
-	for( pifreq = pifconf->ifc_ifcu.ifcu_req, i = 0;
-	     pifconf->ifc_len > i;)
-	{
-		if ( iface == (char*)pifreq->ifr_name )
-		{
-			memcpy( &ifr, pifreq, sizeof(struct ifreq));
-
-			if ( ioctl( sock, SIOCGIFADDR, &ifr ) != -1 )
-			{
-				s = inet_ntoa(((sockaddr_in*)&ifr.ifr_broadaddr)->sin_addr);
-			}
-
-			break;
-		}
-
-#ifdef HAVE_SA_LEN
-		i += IFNAMSIZ + pifreq->ifr_addr.sa_len;
-#else
-		i += sizeof( struct ifreq );
-#endif
-		pifreq = (struct ifreq *)((caddr_t)pifconf->ifc_ifcu.ifcu_req + i);
-	}
-
-	close(sock);
-	free(pifconf);
-#endif
-	return s;
-}
-
 /** */
 int CSocket::GetFreeSendBufferSize()
 {
-	int tot, free;
+	int tot, free = 0;
 #ifndef WIN32
 	int unsent;
 #endif
 	socklen_t ilen;
 
-	free = 0;
-
 	if ( iHandle == INVALID_SOCKET )
 	{
 		return free;
@@ -1014,52 +218,6 @@ int CSocket::IsConnect()
 	FD_CLR(iHandle, &wset);
 	FD_CLR(iHandle, &eset);
 
-#ifdef HAVE_SSL
-	if ( (err != -1) && (m_eSocketMode != esmSOCKET) && (m_bSSLInit == FALSE))
-	{
-		if ( m_eSocketMode == esmSSLCLIENT )
-		{
-			err = SSL_connect(m_pSSL);
-		}
-		else if ( m_eSocketMode == esmSSLSERVER )
-		{
-			err = SSL_accept(m_pSSL);
-		}
-
-		if ( err != 1 )
-		{
-			ERR_print_errors_fp(stderr);
-			err = SSL_get_error(m_pSSL,err);
-
-			if ( (err != SSL_ERROR_WANT_READ) && (err != SSL_ERROR_WANT_WRITE) )
-			{
-				err = -1;
-				ERR_print_errors_fp(stderr);
-			}
-			else
-			{
-				err = 0;
-			}
-		}
-		else
-		{
-			printf("Connected with %s encryption '%s'\n",
-				SSL_get_cipher(m_pSSL),
-				SSL_get_cipher_version(m_pSSL));
-
-			if ( SSL_get_peer_certificate(m_pSSL) == NULL )
-			{
-				printf("No certificate\n");
-			}
-			else if ( (err=SSL_get_verify_result(m_pSSL)) != X509_V_OK )
-			{
-				printf("Certificate verify failed: '%s'\n",X509_verify_cert_error_string(err));
-			}
-			m_bSSLInit = TRUE;
-		}
-	}
-#endif
-
 	return err;
 }
 
@@ -1069,7 +227,7 @@ int CSocket::SocketError()
 	int err = 0;
 	socklen_t ilen;
 
-	if ( iHandle == -1 )
+	if ( iHandle == INVALID_SOCKET )
 	{
 		return err;
 	}
@@ -1088,9 +246,9 @@ int CSocket::SocketError()
 eConnectState CSocket::Connect( CString Host, bool bAsync )
 {
 	unsigned int port;
-	CString s = "";
+	CString s;
 
-	ParseHost( Host, s, port );
+	CNetAddr::ParseHost( Host, s, port );
 
 	// set default port
 	if ( port == 0 )
@@ -1107,7 +265,6 @@ eConnectState CSocket::Connect( CString Host, int Port, bool bAsync )
 	struct sockaddr_in sin,t_sin;
 	SOCKET sockfd;
 	eAsyncDns ead;
-	int err;
 
 	if ( iHandle != INVALID_SOCKET )
 	{
@@ -1117,7 +274,7 @@ eConnectState CSocket::Connect( CString Host, int Port, bool bAsync )
 	memset((void *)&sin, 0, sizeof(sin));
 	sin.sin_family = AF_INET;
 
-	if ( Host == "" )
+	if ( Host.IsEmpty() )
 	{
 		if ( SocketType == estTCP )
 		{
@@ -1130,21 +287,27 @@ eConnectState CSocket::Connect( CString Host, int Port, bool bAsync )
 	}
 	else
 	{
-		if ( bAsync == FALSE )
-		{
-			// use blocked gethostbyname
-			if ( GetHostByName( Host.Data(), &t_sin ) == FALSE )
-			{
+		/* skip host lookup if it is a valid IP address */
 #ifdef WIN32
-				sError = ext_strerror(WSAGetLastError());
+		sin.sin_addr.s_addr = inet_addr( Host.Data() );
+		if ( sin.sin_addr.s_addr != INADDR_NONE )
 #else
-				sError = hstrerror(h_errno);
+		if ( inet_aton( Host.Data(), &sin.sin_addr ) != 0 )
 #endif
+		{
+			m_sIP = Host;
+		}
+		else if ( bAsync == false )
+		{
+			// use blocked getaddrinfo
+			if ( CNetAddr::GetHostI4( Host.Data(), &t_sin, &sError ) == false )
+			{
 				return ecsERROR;
 			}
 			else
 			{
 		 		memcpy((void *)&sin.sin_addr, (void *)&t_sin.sin_addr, sizeof(t_sin.sin_addr));
+				m_sIP  = inet_ntoa(sin.sin_addr);
 			}
 		}
 		else
@@ -1152,7 +315,7 @@ eConnectState CSocket::Connect( CString Host, int Port, bool bAsync )
 			// use async dns class
 			if ( CAsyncDns::Instance() )
 			{
-				ead = CAsyncDns::Instance()->GetHostByName( Host, &t_sin, &err );
+				ead = CAsyncDns::Instance()->GetHostI4( Host, &t_sin, &sError );
 			}
 			else
 			{
@@ -1165,22 +328,22 @@ eConnectState CSocket::Connect( CString Host, int Port, bool bAsync )
 			}
 			else if ( ead == eadERROR )
 			{
-#ifdef WIN32
-				sError = ext_strerror(err);
-#else
-				sError = hstrerror(err);
-#endif
 				return ecsERROR;
 			}
 			else
 			{
 				memcpy((void *)&sin.sin_addr, (void *)&t_sin.sin_addr, sizeof(t_sin.sin_addr));
+				m_sIP  = inet_ntoa(sin.sin_addr);
 			}
 		}
 	}
 
 	sin.sin_port = htons((unsigned short)Port);
 
+	// update m_sIP
+	m_sIP += ':';
+	m_sIP += CString::number(Port);
+
 	// socket
 	if ( SocketType == estTCP )
 	{
@@ -1212,7 +375,7 @@ eConnectState CSocket::Connect( CString Host, int Port, bool bAsync )
 		return ecsERROR;
 	}
 
-	if ( Host != "" )
+	if ( Host.NotEmpty() )
 	{
 		// connect
 		if ( connect(sockfd, (struct sockaddr *)&sin, sizeof(sin)) != 0 )
@@ -1248,8 +411,24 @@ eConnectState CSocket::Connect( CString Host, int Port, bool bAsync )
 		}
 	}
 
+#if DCLIB_USES_OPENSSL == 1
+	if ( m_eSocketMode == esmFULLSSLCLIENT )
+	{
+		if ( SSL_set_fd(m_pSSL, sockfd) == 0 )
+		{
+			sError  = "CSocket::Connect: SSL_set_fd failed: ";
+			sError += ERR_reason_error_string( ERR_get_error() );
+#ifdef WIN32
+			closesocket(sockfd);
+#else
+			close(sockfd);
+#endif
+			return ecsERROR;
+		}
+	}
+#endif /* DCLIB_USES_OPENSSL */
+	
 	iHandle = sockfd;
-
 	return ecsSUCCESS;
 }
 
@@ -1258,9 +437,10 @@ int CSocket::Disconnect()
 {
 	if ( iHandle != INVALID_SOCKET )
 	{
-#ifdef HAVE_SSL
+#if DCLIB_USES_OPENSSL == 1
 		if ( (m_eSocketMode != esmSOCKET) && (m_pSSL) )
 		{
+			SSL_shutdown(m_pSSL); // FIXME check return value
 			SSL_free(m_pSSL);
 			m_pSSL = 0;
 		}
@@ -1271,7 +451,7 @@ int CSocket::Disconnect()
 #else
 		close(iHandle);
 #endif
-#ifdef HAVE_SSL
+#if DCLIB_USES_OPENSSL == 1
 		if ( (m_eSocketMode != esmSOCKET) && (m_pCTX) )
 		{
 			SSL_CTX_free(m_pCTX);
@@ -1280,7 +460,6 @@ int CSocket::Disconnect()
 #endif
 		m_eSocketMode = esmSOCKET;
 		iHandle       = INVALID_SOCKET;
-		m_bSSLInit    = FALSE;
 	}
 
 	return 0;
@@ -1289,28 +468,24 @@ int CSocket::Disconnect()
 /** */
 int CSocket::Read( char * buffer, int len, int sec_timeout, int usec_timeout )
 {
-	int l,i;
+	int l = 0, i = 0;
 	struct timeval tv;
 	fd_set readset;
-	socklen_t cli_len;
-	struct sockaddr cli_addr;
-
-	cli_len = sizeof(cli_addr);
-
-	l = 0;
-	i = 0;
+	struct sockaddr_in cli_addr;
+	socklen_t cli_len = sizeof(cli_addr);
 
 	if ( (iHandle == INVALID_SOCKET) || (!buffer) || (len <= 0) )
 	{
 		return -1;
 	}
 	
-#ifdef HAVE_SSL
-	if ( (m_eSocketMode == esmSSLCLIENT) || (m_eSocketMode == esmSSLSERVER) )
+#if DCLIB_USES_OPENSSL == 1
+	if ( m_eSocketMode != esmSOCKET )
 	{
 		l = SSL_read(m_pSSL, buffer, len);
 
-		if ( (l <= 0) && ((i=IsConnect()) != -1) )
+		// if ( (l <= 0) && ((i=IsConnect()) != -1) )
+		if ( l <= 0 )
 		{
 //			printf("READ: %d %d\n",l,i);
 
@@ -1319,7 +494,15 @@ int CSocket::Read( char * buffer, int len, int sec_timeout, int usec_timeout )
 			if ( (l != SSL_ERROR_WANT_READ) && (l != SSL_ERROR_WANT_WRITE) )
 			{
 				l = -1;
+				unsigned long sslerr = ERR_peek_error();  // ERR_get_error() if not printed
 				ERR_print_errors_fp(stderr);
+				sError  = "SSL ERROR lib:";
+				sError += ERR_lib_error_string(sslerr);
+				sError += " func:";
+				sError += ERR_func_error_string(sslerr);
+				sError += " reason:";
+				sError += ERR_reason_error_string(sslerr);
+				Disconnect();
 			}
 			else
 			{
@@ -1350,7 +533,7 @@ int CSocket::Read( char * buffer, int len, int sec_timeout, int usec_timeout )
 			// handle udp socket
 			if ( SocketType == estUDP )
 			{
-				if ( (l=recvfrom(iHandle,buffer,len,0,&cli_addr,&cli_len)) < 0 )
+				if ( (l=recvfrom(iHandle,buffer,len,0,(struct sockaddr*)&cli_addr,&cli_len)) < 0 )
 				{
 #ifdef WIN32
 					int e = WSAGetLastError();
@@ -1368,6 +551,10 @@ int CSocket::Read( char * buffer, int len, int sec_timeout, int usec_timeout )
 						i = l = 0;
 					}
 				}
+				else if ( l > 0 )
+				{
+					m_sIP = inet_ntoa( cli_addr.sin_addr );
+				}
 			}
 			// handle tcp socket
 			else if ( SocketType == estTCP )
@@ -1417,17 +604,15 @@ int CSocket::Read( char * buffer, int len, int sec_timeout, int usec_timeout )
 	{
 		if ( l > 0 )
 		{
-			CString s;
-			s = "RECV:" + CString().setNum(l);
+			CString s = "RECV:";
+			s += CString::number(l);
+			CLogFile::Write("dcsocket.log",eltINFO,s);
+			
 			if( l <= len )
 			{
-				char prev = buffer[l];
-				s+="\n";
-				buffer[l]=0;
-				s += buffer;
-				buffer[l] = prev;
+				s.Set(buffer,l);
+				CLogFile::Write("dcsocket.log",eltINFO,s);
 			}
-			CLogFile::Write("dcsocket.log",eltINFO,s);
 		}
 	}
 
@@ -1472,12 +657,13 @@ int CSocket::Write( const unsigned char * buffer, int len, int sec_timeout, int
 		return -1;
 	}
 
-#ifdef HAVE_SSL
-	if ( (m_eSocketMode == esmSSLCLIENT) || (m_eSocketMode == esmSSLSERVER) )
+#if DCLIB_USES_OPENSSL == 1
+	if ( m_eSocketMode != esmSOCKET )
 	{
 		i = SSL_write(m_pSSL, buffer, len);
-
-		if ( (i == -1) && (IsConnect() != -1) )
+		
+		// if ( (i <= 0) && (IsConnect() != -1) )
+		if ( i <= 0 )
 		{
 //			printf("WRITE: %d %d\n",i,IsConnect());
 
@@ -1486,7 +672,15 @@ int CSocket::Write( const unsigned char * buffer, int len, int sec_timeout, int
 			if ( (i != SSL_ERROR_WANT_READ) && (i != SSL_ERROR_WANT_WRITE) )
 			{
 				i = -1;
+				unsigned long sslerr = ERR_peek_error();  // ERR_get_error() if not printed
 				ERR_print_errors_fp(stderr);
+				sError  = "SSL ERROR lib:";
+				sError += ERR_lib_error_string(sslerr);
+				sError += " func:";
+				sError += ERR_func_error_string(sslerr);
+				sError += " reason:";
+				sError += ERR_reason_error_string(sslerr);
+				Disconnect();
 			}
 			else
 			{
@@ -1516,17 +710,9 @@ int CSocket::Write( const unsigned char * buffer, int len, int sec_timeout, int
 
 		if ( i > 0 )
 		{
-			// handle tcp socket
-			if ( SocketType == estTCP )
-			{
-				i = send( iHandle, (const char*)buffer, len, MSG_NOSIGNAL );
-			}
-			else
-			{
-				i = send( iHandle, (const char*)buffer, len, MSG_NOSIGNAL );
-			}
+			i = send( iHandle, (const char*)buffer, len, MSG_NOSIGNAL );
 
-			// send error or wrong socket type
+			// send error
 			if ( i < 0 )
 			{
 #ifdef WIN32
@@ -1555,11 +741,7 @@ int CSocket::Write( const unsigned char * buffer, int len, int sec_timeout, int
 		{
 			i = SocketError();
 
-			if ( i == 0 )
-			{
-				i = 0;
-			}
-			else
+			if ( i != 0 )
 			{
 				sError = ext_strerror(i);
 				i = -1;
@@ -1573,12 +755,11 @@ int CSocket::Write( const unsigned char * buffer, int len, int sec_timeout, int
 	{
 		if ( i > 0 )
 		{
-			CString s;
-			char prev = ((char*)buffer)[i];
-			s = "SEND:" + CString().setNum(len)+"\n";
-			((char*)buffer)[i]=0;
-			s += (char*)buffer;
-			((char*)buffer)[i] = prev;
+			CString s = "SEND:";
+			s += CString::number(len);
+			CLogFile::Write("dcsocket.log",eltINFO,s);
+			
+			s.Set( (const char*)buffer ,i );
 			CLogFile::Write("dcsocket.log",eltINFO,s);
 		}
 	}
@@ -1595,11 +776,9 @@ int CSocket::Accept()
 	struct timeval tv;
 	fd_set readset;
 	struct sockaddr_in serv_addr;
-	SOCKET s = -1;
+	SOCKET s = INVALID_SOCKET;
 	int i;
-	socklen_t sin_size;
-
-	sin_size = sizeof(struct sockaddr_in);
+	socklen_t sin_size = sizeof(struct sockaddr_in);
 
 	if ( iHandle == INVALID_SOCKET )
 	{
@@ -1682,12 +861,16 @@ int CSocket::Listen( int port, CString ip )
 	serv_addr.sin_family = AF_INET;
 	serv_addr.sin_port = htons((unsigned short)port);
 
-	if ( ip != "" )
+	if ( ip.NotEmpty() )
 	{
-		serv_addr.sin_addr.s_addr = inet_addr(ip.Data());
-
-		if ( serv_addr.sin_addr.s_addr == 255 )
+#ifdef WIN32
+		serv_addr.sin_addr.s_addr = inet_addr( ip.Data() );
+		if ( serv_addr.sin_addr.s_addr == INADDR_NONE )
+#else
+		if ( inet_aton( ip.Data(), &serv_addr.sin_addr ) == 0 )
+#endif
 		{
+			sError = "Invalid IP address";
 			return -1;
 		}
 	}
@@ -1720,8 +903,24 @@ int CSocket::Listen( int port, CString ip )
 		return -1;
 	}
 
-	iHandle = sockfd;
+#if DCLIB_USES_OPENSSL == 1
+	if ( m_eSocketMode == esmFULLSSLSERVER )
+	{
+		if ( SSL_set_fd(m_pSSL, sockfd) == 0 )
+		{
+			sError  = "CSocket::Listen: SSL_set_fd failed: ";
+			sError += ERR_reason_error_string( ERR_get_error() );
+#ifdef WIN32
+			closesocket(sockfd);
+#else
+			close(sockfd);
+#endif
+			return -1;
+		}
+	}
+#endif /* DCLIB_USES_OPENSSL */
 
+	iHandle = sockfd;
 	return 0;
 }
 
@@ -1731,14 +930,9 @@ bool CSocket::GetPeerName( CString * host, int * port )
 	struct sockaddr_in addr;
 	socklen_t sin_size;
 
-	if ( iHandle == INVALID_SOCKET )
+	if ( (iHandle == INVALID_SOCKET) || (!host) || (!port) )
 	{
-		return FALSE;
-	}
-
-	if ( (!host) || (!port) )
-	{
-		return FALSE;
+		return false;
 	}
 
 	sin_size = sizeof(struct sockaddr_in);
@@ -1746,33 +940,39 @@ bool CSocket::GetPeerName( CString * host, int * port )
 	if ( getpeername( iHandle, (struct sockaddr*)&addr, &sin_size ) == -1 )
 	{
 		sError = ext_strerror(SocketError());
-		return FALSE;
+		return false;
 	}
 
 	*host = inet_ntoa(addr.sin_addr);
 	*port = ntohs(addr.sin_port);
 
-	return TRUE;
+	return true;
 }
 
 /** */
+#if DCLIB_USES_OPENSSL == 1
 bool CSocket::ChangeSocketMode( eSocketMode mode, CString cert, CString key )
+#else
+bool CSocket::ChangeSocketMode( eSocketMode mode, CString /* cert */, CString /* key */ )
+#endif
 {
-	bool res = FALSE;
+	bool res = false;
 
 	switch(mode)
 	{
 		case esmSOCKET:
 		{
 			m_eSocketMode = mode;
-			res = TRUE;
+			res = true;
 			break;
 		}
-#ifdef HAVE_SSL
+#if DCLIB_USES_OPENSSL == 1
 		case esmSSLCLIENT:
 		case esmSSLSERVER:
+		case esmFULLSSLCLIENT:
+		case esmFULLSSLSERVER:
 		{
-			if ( ((cert == "") || (key == "")) && (mode == esmSSLSERVER) )
+			if ( (cert.IsEmpty() || key.IsEmpty()) && ((mode == esmSSLSERVER) || (mode == esmFULLSSLSERVER)) )
 			{
 				printf("no cert/key available\n");
 				return res;
@@ -1780,29 +980,54 @@ bool CSocket::ChangeSocketMode( eSocketMode mode, CString cert, CString key )
 
 			if ( m_eSocketMode == esmSOCKET )
 			{
-				CSSL cssl;
-
-				if ( mode == esmSSLCLIENT )
+				if ( mode == esmFULLSSLCLIENT )
 				{
-					if ( (m_pCTX = cssl.InitClientCTX()) == 0 )
+					m_pCTX = CSSL::NewTLSv1ClientCTX();
+					
+					if ( m_pCTX == 0 )
+					{
+						printf("CSocket::ChangeSocketMode NewTLSv1ClientCTX failed\n");
+						return res;
+					}
+				}
+				else if ( mode == esmFULLSSLSERVER )
+				{
+					m_pCTX = CSSL::NewTLSv1ServerCTX();
+					
+					if ( m_pCTX == 0 )
+					{
+						printf("CSocket::ChangeSocketMode NewTLSv1ServerCTX failed\n");
+						return res;
+					}
+				}
+				else if ( mode == esmSSLCLIENT )
+				{
+					m_pCTX = CSSL::InitClientCTX();
+					
+					if ( m_pCTX == 0 )
 					{
 						printf("InitClientCTX failed\n");
 						return res;
 					}
 				}
-				else
+				else 
 				{
-					if ( (m_pCTX = cssl.InitServerCTX()) == 0 )
+					m_pCTX = CSSL::InitServerCTX();
+					
+					if ( m_pCTX == 0 )
 					{
 						printf("InitServerCTX failed\n");
 						return res;
 					}
-					
-					if ( cssl.LoadCertificates(m_pCTX, cert.Data(), key.Data() ) == FALSE )
+				}
+				
+				if ( cert.NotEmpty() && key.NotEmpty() )
+				{
+					if ( CSSL::LoadCertificates( m_pCTX, cert.Data(), key.Data() ) == false )
 					{
 						SSL_CTX_free(m_pCTX);
 						m_pCTX = 0;
-
+						
 						printf("load cert/key failed\n");
 						return res;
 					}
@@ -1815,12 +1040,19 @@ bool CSocket::ChangeSocketMode( eSocketMode mode, CString cert, CString key )
 					printf("SSL_new failed\n");
 					SSL_CTX_free(m_pCTX);
 					m_pCTX = 0;
-					SSL_free(m_pSSL);
-					m_pSSL = 0;
 
 					return res;
 				}
 
+				if ( (mode == esmSSLSERVER) || (mode == esmFULLSSLSERVER) )
+				{
+					SSL_set_accept_state(m_pSSL);
+				}
+				else
+				{
+					SSL_set_connect_state(m_pSSL);
+				}
+
 				if ( SSL_set_fd(m_pSSL, iHandle) == 0 )
 				{
 					printf("SSL_set_fd failed\n");
@@ -1833,7 +1065,7 @@ bool CSocket::ChangeSocketMode( eSocketMode mode, CString cert, CString key )
 				}
 
 				m_eSocketMode = mode;
-				res = TRUE;
+				res = true;
 			}
 			else
 			{
@@ -1922,3 +1154,88 @@ CString CSocket::ext_strerror( int err )
 #endif
 	return strerror(err);
 }
+
+/** */
+CString CSocket::GetSSLVersion()
+{
+	CString result;
+#if DCLIB_USES_OPENSSL == 1
+	if ( m_pSSL )
+	{
+		result = SSL_get_cipher_version(m_pSSL);
+	}
+#endif
+	return result;
+}
+
+/** */
+CString CSocket::GetSSLCipher()
+{
+	CString result;
+#if DCLIB_USES_OPENSSL == 1
+	if ( m_pSSL )
+	{
+		result = SSL_get_cipher(m_pSSL);
+	}
+#endif
+	return result;
+}
+
+/** */
+CString CSocket::VerifyPeerCertificate()
+{
+	CString result;
+#if DCLIB_USES_OPENSSL == 1
+	if ( m_pSSL )
+	{
+		if ( SSL_get_peer_certificate(m_pSSL) == NULL )
+		{
+			result = "No certificate";
+		}
+		else
+		{
+			int certerr = SSL_get_verify_result(m_pSSL);
+			
+			if ( certerr == X509_V_OK )
+			{
+				result = "Certificate verified";
+			}
+			else
+			{
+				result  = "Certificate verify failed: ";
+				result += X509_verify_cert_error_string(certerr);
+			}
+		}
+	}
+#endif
+	return result;
+}
+
+/** */
+int CSocket::SysInit()
+{
+#ifdef WIN32
+	/* According to the docs WSAStartup()
+	 * will fail if the requested version could not be
+	 * provided, plus 2.2 is the latest version and that
+	 * can even be installed on Windows 95.
+	 */
+	WORD wVersionRequested = MAKEWORD( 2, 2 );
+	WSADATA wsaData;
+	return WSAStartup( wVersionRequested, &wsaData );
+#else
+	/* nothing to do */
+	return 0;
+#endif
+}
+
+/** */
+int CSocket::SysDeInit()
+{
+#ifdef WIN32
+	return WSACleanup();
+#else
+	/* nothing to do */
+	return 0;
+#endif
+}
