@@ -2,7 +2,7 @@
                           cstring.cpp  -  description
                              -------------------
     begin                : Fri Sep 21 2001
-    copyright            : (C) 2001-2003 by Mathias Kster
+    copyright            : (C) 2001-2003 by Mathias Küster
     email                : mathen@users.berlios.de
  ***************************************************************************/
 
@@ -15,6 +15,8 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "cstring.h"
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -25,8 +27,11 @@
 #include <ctype.h>
 #include <string.h>
 
-#include <dclib/dcos.h>
-#include "cstring.h"
+/* for testing both code paths */
+/* ifdef HAVE_STRCASESTR
+ * undef HAVE_STRCASESTR
+ * endif
+ */
 
 CString::CString()
 {
@@ -47,13 +52,20 @@ CString::CString( const char ch )
 	set(&ch,1);
 }
 
-CString::CString( const CString & stringSrc ) : CObject()
+CString::CString( const CString & stringSrc )
 {
 	Init();
 
 	set(stringSrc.m_szBuffer,stringSrc.m_nStringLength);
 }
 
+CString::CString( char * buffer, long length, long bufsize )
+{
+	m_szBuffer = buffer;
+	m_nStringLength = length;
+	m_nBufferSize = bufsize;
+}
+
 CString::~CString()
 {
 	if (m_szBuffer)
@@ -85,7 +97,7 @@ void CString::Empty()
 /** */
 unsigned char CString::GetHash( long value ) const
 {
-	if ( (!IsEmpty()) && (value < m_nStringLength) )
+	if ( (m_nStringLength > 0) && (value < m_nStringLength) )
 		return ((unsigned char)(m_szBuffer[value]&0xFF));
 	else
 		return 0;
@@ -94,20 +106,15 @@ unsigned char CString::GetHash( long value ) const
 /** */
 long CString::Find( const char ch, long nStart ) const
 {
-	long i;
-
-	if ( IsEmpty() )
+	if ( m_nStringLength == 0 )
 	{
 		return -1;
 	}
 
-	if ( nStart == -1 )
-		nStart = 0;
-
 	if ( nStart > m_nStringLength )
 		return -1;
 
-	for(i=nStart;i<m_nStringLength;i++)
+	for ( long i = nStart; i < m_nStringLength; ++i )
 	{
 		if ( m_szBuffer[i] == ch )
 			return i;
@@ -119,16 +126,12 @@ long CString::Find( const char ch, long nStart ) const
 /** */
 long CString::Find( const char * sz, long nStart, bool cs ) const
 {
-	long i;
-	long len;
-	long ret;
-
 	if ( !cs )
 	{
 		return FindCase(sz,nStart);
 	}
 
-	if ( IsEmpty() )
+	if ( m_nStringLength == 0 )
 	{
 		if ( sz != 0 )
 			return -1;
@@ -139,79 +142,61 @@ long CString::Find( const char * sz, long nStart, bool cs ) const
 	if ( !sz )
 		return -1;
 
-	if ( nStart == -1 )
-		nStart = 0;
-
-	len = strlen(sz);
+	const long len = strlen(sz);
 
 	if ( (nStart+len) > m_nStringLength )
 		return -1;
 
-	for(i=nStart,ret=-1;i<=(m_nStringLength-len);i++)
+	const char * location = strstr( m_szBuffer + nStart, sz );
+	
+	if ( location == NULL )
 	{
-		if ( strncmp(sz,m_szBuffer+i,len) == 0 )
-		{
-			ret = i; //-index;
-			break;
-		}
+		return -1;
+	}
+	else
+	{
+		return location - m_szBuffer;
 	}
-
-	return ret;
 }
 
 /** */
 long CString::Find( const CString & string, long nStart, bool cs ) const
 {
-	long i;
-	long len;
-	long ret;
-	char * sz;
-
 	if ( !cs )
 	{
 		return FindCase(string,nStart);
 	}
 
-	sz  = string.Data();
-	len = string.Length();
-
-	if ( IsEmpty() )
+	if ( m_nStringLength == 0 )
 	{
-		if ( sz != 0 )
-			return -1;
-		else
+		if ( string.Length() == 0 )
 			return 0;
+		else
+			return -1;
 	}
 
-	if ( !sz )
+	if ( string.Data() == 0 )
 		return -1;
 
-	if ( nStart == -1 )
-		nStart = 0;
-
-	if ( (nStart+len) > m_nStringLength )
+	if ( (nStart+string.Length()) > m_nStringLength )
 		return -1;
 
-	for(i=nStart,ret=-1;i<=(m_nStringLength-len);i++)
+	const char * location = strstr( m_szBuffer + nStart, string.Data() );
+	
+	if ( location == NULL )
 	{
-		if ( strncmp(sz,m_szBuffer+i,len) == 0 )
-		{
-			ret = i; //-index;
-			break;
-		}
+		return -1;
+	}
+	else
+	{
+		return location - m_szBuffer;
 	}
-
-	return ret;
 }
 
 /** */
 long CString::FindCase( const char * sz, long nStart ) const
 {
-	long i;
-	long len;
-	long ret;
-
-	if ( IsEmpty() )
+	if ( m_nStringLength == 0 )
 	{
 		if ( sz != 0 )
 			return -1;
@@ -222,15 +207,27 @@ long CString::FindCase( const char * sz, long nStart ) const
 	if ( !sz )
 		return -1;
 
-	if ( nStart == -1 )
-		nStart = 0;
-
-	len = strlen(sz);
+	const long len = strlen(sz);
 
 	if ( (nStart+len) > m_nStringLength )
 		return -1;
 
-	for(i=nStart,ret=-1;i<=(m_nStringLength-len);i++)
+#ifdef HAVE_STRCASESTR
+
+	const char * location = strcasestr( m_szBuffer + nStart, sz );
+	
+	if ( location == NULL )
+	{
+		return -1;
+	}
+	else
+	{
+		return location - m_szBuffer;
+	}
+
+#else // HAVE_STRCASESTR
+
+	for ( long i = nStart; i <= (m_nStringLength-len); ++i )
 	{
 #ifdef WIN32
 		if ( _strnicmp(sz,m_szBuffer+i,len) == 0 )
@@ -238,26 +235,21 @@ long CString::FindCase( const char * sz, long nStart ) const
 		if ( strncasecmp(sz,m_szBuffer+i,len) == 0 )
 #endif
 		{
-			ret = i-nStart;
-			break;
+			return i;
 		}
 	}
-
-	return ret;
+	
+	return -1;
+#endif // HAVE_STRCASESTR
 }
 
 /** */
 long CString::FindCase( const CString & string, long nStart ) const
 {
-	long i;
-	long len;
-	long ret;
-	char * sz;
+	const char * sz = string.Data();
+	const long len  = string.Length();
 
-	sz  = string.Data();
-	len = string.Length();
-
-	if ( IsEmpty() )
+	if ( m_nStringLength == 0 )
 	{
 		if ( sz != 0 )
 			return -1;
@@ -268,13 +260,24 @@ long CString::FindCase( const CString & string, long nStart ) const
 	if ( !sz )
 		return -1;
 
-	if ( nStart == -1 )
-		nStart = 0;
-
 	if ( (nStart+len) > m_nStringLength )
 		return -1;
+#ifdef HAVE_STRCASESTR
 
-	for(i=nStart,ret=-1;i<=(m_nStringLength-len);i++)
+	const char * location = strcasestr( m_szBuffer + nStart, sz );
+	
+	if ( location == NULL )
+	{
+		return -1;
+	}
+	else
+	{
+		return location - m_szBuffer;
+	}
+
+#else // HAVE_STRCASESTR
+
+	for ( long i = nStart; i <= (m_nStringLength-len); ++i )
 	{
 #ifdef WIN32
 		if ( _strnicmp(sz,m_szBuffer+i,len) == 0 )
@@ -282,20 +285,19 @@ long CString::FindCase( const CString & string, long nStart ) const
 		if ( strncasecmp(sz,m_szBuffer+i,len) == 0 )
 #endif
 		{
-			ret = i-nStart;
-			break;
+			return i;
 		}
 	}
+	
+	return -1;
 
-	return ret;
+#endif // HAVE_STRCASESTR
 }
 
 /** */
 long CString::FindRev( char ch, long nStart ) const
 {
-	long i;
-
-	if ( IsEmpty() )
+	if ( m_nStringLength == 0 )
 		return -1;
 
 	if ( nStart == -1 )
@@ -304,7 +306,7 @@ long CString::FindRev( char ch, long nStart ) const
 	if ( nStart > (m_nStringLength+1) )
 		return -1;
 
-	for(i=nStart;i>=0;i--)
+	for ( long i = nStart; i >= 0; --i )
 	{
 		if (m_szBuffer[i]==ch)
 			return i;
@@ -316,41 +318,34 @@ long CString::FindRev( char ch, long nStart ) const
 /** */
 long CString::FindRev( const CString & string ) const
 {
-	long i;
-	long len;
-	long ret=-1;
-	char * sz;
+	const long len = string.Length();
 
-	sz  = string.Data();
-	len = string.Length();
-
-	if ( IsEmpty() )
+	if ( m_nStringLength == 0 )
 	{
-		if ( sz != 0 )
-			return -1;
-		else
+		if ( len == 0 )
 			return 0;
+		else
+			return -1;
 	}
 
-	if ( !sz )
+	if ( len == 0 )
 		return -1;
 
-	for(i=(m_nStringLength-len);i>=0;i--)
+	for ( long i = (m_nStringLength-len); i >= 0; --i )
 	{
 		if ( string == CString(m_szBuffer).Mid(i,len) )
 		{
-			ret = i; //-index;
-			break;
+			return i;
 		}
 	}
 
-	return ret;
+	return -1;
 }
 
 /** */
 CString CString::Mid( long nFirst, long nCount ) const
 {
-	CString tmp = "";
+	CString ret;
 
 	// added_by_frg for nFirst < 0 case:
 	// which is needed in all occasions where we use str.Right(n) where n > str.Length()
@@ -361,69 +356,57 @@ CString CString::Mid( long nFirst, long nCount ) const
 	}
 	// addition_end
 
-	if ( IsEmpty() )
-		return tmp;
+	if ( m_nStringLength == 0 )
+		return ret;
 	if ( nCount == -1 )
 		nCount = m_nStringLength-nFirst;
 	if ( nCount <= 0 )
-		return tmp;
+		return ret;
 	if( nFirst > m_nStringLength )
-		return tmp;
+		return ret;
 
 	if( (nFirst+nCount) > m_nStringLength )
-		return tmp;
+		return ret;
 
-	tmp.set(m_szBuffer+nFirst,nCount);
+	ret.set(m_szBuffer+nFirst,nCount);
 
-	return tmp;
+	return ret;
 }
 
 /** */
 CString CString::Section( const char sep, int start, int end ) const
 {
 	int i,si,ei;
-	CString tmp = "";
 
-	if ( IsEmpty() )
-		return tmp;
+	if ( m_nStringLength == 0 )
+		return CString();
 
-	for(i=0,si=0;i<start&&si!=-1;i++,si++)
+	for(i=0,si=0;i<start;i++,si++)
 	{
 		si = Find(sep,si);
+		if ( si == -1 )
+			break;
 	}
 
 	if ( si == -1 )
-		return tmp;
+		return CString();
 
-	for(ei=si;i<=end&&ei!=-1;i++,ei++)
+	for(ei=si;i<=end;i++,ei++)
 	{
 		ei = Find(sep,ei);
+		if ( ei == -1 )
+			break;
 	}
 
-	if ( (si==-1) || (ei==-1) )
-		return tmp;
+	if ( ei == -1 )
+		ei = m_nStringLength + 1;
+
+	/* if ( (si==-1) || (ei==-1) )
+		return CString(); */
 
 	return Mid(si,ei-si-1);
 }
 
-/** */
-void CString::Append( const CString & string )
-{
-	CString tmp(string);
-
-	add(tmp.Data(),tmp.Length());
-}
-
-/** */
-void CString::Append( const char ch )
-{
-	char d[2];
-
-	d[0]=ch;
-	d[1]=0;
-	add(d,1);
-}
-
 /** */
 bool CString::IsEmpty() const
 {
@@ -431,7 +414,7 @@ bool CString::IsEmpty() const
 }
 
 /** */
-CString CString::setNum( const int n )
+CString CString::number( const int n )
 {
 	char c[50];
 
@@ -445,7 +428,7 @@ CString CString::setNum( const int n )
 }
 
 /** */
-CString CString::setNum( const unsigned int n )
+CString CString::number( const unsigned int n )
 {
 	char c[50];
 
@@ -459,7 +442,7 @@ CString CString::setNum( const unsigned int n )
 }
 
 /** */
-CString CString::setNum( const long n )
+CString CString::number( const long n )
 {
 	char c[50];
 
@@ -473,7 +456,7 @@ CString CString::setNum( const long n )
 }
 
 /** */
-CString CString::setNum( const unsigned long n )
+CString CString::number( const unsigned long n )
 {
 	char c[50];
 
@@ -487,12 +470,12 @@ CString CString::setNum( const unsigned long n )
 }
 
 /** */
-CString CString::setNum( const long long n )
+CString CString::number( const long long n )
 {
 	char c[50];
 
 #ifdef WIN32
-	_snprintf(c,50,"%lld",n);
+	_i64toa( n, c, 10 );
 #else
 	snprintf(c,50,"%lld",n);
 #endif
@@ -501,7 +484,7 @@ CString CString::setNum( const long long n )
 }
 
 /** */
-CString CString::setNum( const ulonglong n )
+CString CString::number( const ulonglong n )
 {
 	char c[50];
 
@@ -515,7 +498,7 @@ CString CString::setNum( const ulonglong n )
 }
 
 /** */
-CString CString::setNum( const double n, const int p )
+CString CString::number( const double n, const int p )
 {
 	char c[50];
 
@@ -530,37 +513,28 @@ CString CString::setNum( const double n, const int p )
 /** */
 ulonglong CString::asULL( int base ) const
 {
-	ulonglong res;
-	char *tail=0;
-
-	if ( IsEmpty() )
+	if ( m_nStringLength == 0 )
 	{
 		return 0;
 	}
 
-#ifdef WIN32
-	res=strtoull(Data(),&tail,base);
+#if defined(HAVE_STRTOULL)
+	return strtoull(Data(),NULL,base);
+#elif defined(HAVE__STRTOUI64)
+	return _strtoui64(Data(),NULL,base);
+#elif defined(HAVE_STRTOUQ)
+	return strtouq(Data(),NULL,base);
+#elif defined(__arch64__)
+ 	return strtoul(Data(),NULL,base);
 #else
-	#ifdef HAVE_STRTOULL
-	res=strtoull(Data(),&tail,base);
-	#elif HAVE_STRTOUQ
-	res=strtouq(Data(),&tail,base);
-	#elif defined(__arch64__) 
- 	res=strtoul(Data(),&tail,base);
-	#else
-	#error "You don't have strtoull or strtouq function !"
-	#endif
+	#error "No strtoull _strtoui64 strtouq function available"
 #endif
-
-	if((res==0)&&(errno!=0)) {}
-
-	return res;
 }
 
 /** */
 unsigned int CString::asUINT( int base ) const
 {
-	return (unsigned int)asLONG(base);
+	return (unsigned int)asULONG(base);
 }
 
 /** */
@@ -572,135 +546,197 @@ int CString::asINT( int base ) const
 /** */
 long CString::asLONG( int base ) const
 {
-	unsigned int res;
-	char *tail=0;
-
-	if ( IsEmpty() )
+	if ( m_nStringLength == 0 )
 	{
 		return 0;
 	}
 
-	res=strtol(Data(),&tail,base);
+	return strtol(Data(),NULL,base);
+}
 
-	if((res==0)&&(errno!=0)) {}
-
-	return res;
+unsigned long CString::asULONG( int base ) const
+{
+	if ( m_nStringLength == 0 )
+	{
+		return 0;
+	}
+	
+	return strtoul( Data(), NULL, base );
 }
 
 long long CString::asLONGLONG( int base ) const
 {
-	long long res;
-	char *tail=0;
-	
-	if ( IsEmpty() )
+	if ( m_nStringLength == 0 )
 	{
 		return 0;
 	}
-	
-	res=strtoll(Data(),&tail,base);
-	
-	if((res==0)&&(errno!=0)) {}
-	
-	return res;
+
+#if defined(HAVE_STRTOLL)
+	return strtoll(Data(),NULL,base);
+#elif defined(HAVE__STRTOI64)
+	return _strtoi64(Data(),NULL,base);
+#elif defined(HAVE_STRTOQ)
+	return strtoq(Data(),NULL,base);
+#elif defined(__arch64__)
+	return strtol(Data(),NULL,base);
+#else
+	#error "No strtoll _strtoi64 strtoq function available"
+#endif
 }
 
 /** */
 double CString::asDOUBLE() const
 {
-	char *endptr = 0;
-
-	if ( IsEmpty() )
+	if ( m_nStringLength == 0 )
 	{
 		return 0;
 	}
 
-	return strtod(Data(),&endptr);
+	return strtod(Data(),NULL);
 }
 
 /** */
-CString CString::ToUpper()
+CString CString::ToUpper() const
 {
-	CString ret="";
-	long i=0;
+	if ( m_nStringLength == 0 )
+		return CString();
 
-	if ( IsEmpty() )
-		return ret;
-
-	for(;i<m_nStringLength;i++)
+	char * resbuf = (char*) malloc(m_nStringLength+1);
+	
+	if ( !resbuf )
 	{
-		ret += (const char)(toupper(m_szBuffer[i])&0xFF);
+		perror("CString::ToUpper: malloc");
+		return CString();
 	}
 
-	return ret;
+	for ( long i = 0; i < m_nStringLength; ++i )
+	{
+		resbuf[i] = (const char)(toupper(m_szBuffer[i])&0xFF);
+	}
+
+	resbuf[m_nStringLength] = 0;
+
+	return CString( resbuf, m_nStringLength, m_nStringLength+1 );
 }
 
 /** */
-CString CString::ToLower()
+CString CString::ToLower() const
 {
-	CString ret="";
-	long i=0;
+	if ( m_nStringLength == 0 )
+		return CString();
 
-	if ( IsEmpty() )
-		return ret;
-
-	for(;i<m_nStringLength;i++)
+	char * resbuf = (char*) malloc(m_nStringLength+1);
+	
+	if ( !resbuf )
 	{
-		ret += (const char)(tolower(m_szBuffer[i])&0xFF);
+		perror("CString::ToLower: malloc");
+		return CString();
 	}
 
-	return ret;
+	for ( long i = 0; i < m_nStringLength; ++i )
+	{
+		resbuf[i] = (const char)(tolower(m_szBuffer[i])&0xFF);
+	}
+
+	resbuf[m_nStringLength] = 0;
+
+	return CString( resbuf, m_nStringLength, m_nStringLength+1 );
 }
 
 /** */
 CString CString::Replace( CString src, CString string ) const
 {
-	long i,t;
-	CString tmp = "";
+	long i = 0, t = 0;
+	CString ret;
 
-	t = i = 0;
-	while ( (i=Find(src.Data(),i)) != -1 )
+	while ( (i=Find(src,i)) != -1 )
 	{
-		tmp += Mid(t,i-t);
-		tmp += string;
+		ret += Mid(t,i-t);
+		ret += string;
 		i+=src.Length();
 		t = i;
 	}
 
-	tmp += Mid(t,m_nStringLength-t);
+	ret += Mid(t,m_nStringLength-t);
 
-	return tmp;
+	return ret;
+}
+
+/** */
+void CString::Swap( const char before, const char after )
+{
+	for ( long i = 0; i < m_nStringLength; i++ )
+	{
+		if ( m_szBuffer[i] == before )
+		{
+			m_szBuffer[i] = after;
+		}
+	}
 }
 
 /** */
 void CString::set( const char * sz, long nLength )
 {
 	long l;
-
-	Empty();
+	char * temp;
 
 	if ( !sz )
+	{
+		Empty();
 		return;
+	}
 
 	if( nLength == -1 )
 		l = strlen(sz);
 	else
 		l = nLength;
 
-	if (l<=0)
+	if ( l <= 0 )
+	{
+		Empty();
 		return;
+	}
+
+	if ( (m_szBuffer) && (sz >= m_szBuffer) && ( sz <= m_szBuffer+m_nBufferSize ) )
+	{
+		temp = (char*) malloc( l );
+		if ( temp == 0 )
+		{
+			perror("CString::set malloc ");
+			return;
+		}
+		memcpy(temp,sz,l);
+	}
+	else
+	{
+		temp = (char*) sz;
+	}
+
+	Empty();
 
 	m_szBuffer = (char*) malloc(l+1); //new char[l+1];
 
 	if ( m_szBuffer == 0 )
 	{
 		printf("CString::set malloc [%ld]: %s\n",l+1,strerror(errno));
+		
+		if ( temp != sz )
+		{
+			free(temp);
+		}
+		
 		return;
 	}
 
-	memcpy(m_szBuffer,sz,l);
+	memcpy(m_szBuffer,temp,l);
 	m_szBuffer[l]   = 0;
 	m_nStringLength = l;
 	m_nBufferSize   = l+1;
+	
+	if ( temp != sz )
+	{
+		free(temp);
+	}
 }
 
 /** */
@@ -708,6 +744,7 @@ void CString::add( const char * sz, long nLength )
 {
 	long l,len,i;
 	char *p;
+	char * temp;
 
 	if (!sz)
 		return;
@@ -726,6 +763,23 @@ void CString::add( const char * sz, long nLength )
 	}
 	else
 	{
+		// if pointers overlap, copy input before realloc
+		// which also means memcpy is safe (do not need memmove)
+		if ( (sz >= m_szBuffer) && (sz <= m_szBuffer+m_nBufferSize) )
+		{
+			temp = (char*) malloc( l );
+			if ( temp == 0 )
+			{
+				perror("CString::add malloc ");
+				return;
+			}
+			memcpy(temp,sz,l);
+		}
+		else
+		{
+			temp = (char*) sz;
+		}
+
 		len = m_nStringLength;
 
 		if((l+len+1)>m_nBufferSize)
@@ -735,7 +789,13 @@ void CString::add( const char * sz, long nLength )
 
 			if ( p == 0 )
 			{
-				perror("CString::add realloc");
+				perror("CString::add realloc ");
+				
+				if ( temp != sz )
+				{
+					free(temp);
+				}
+				
 				return;
 			}
 
@@ -744,62 +804,66 @@ void CString::add( const char * sz, long nLength )
 		}
 
 		memset(m_szBuffer+len+l,0,1);
-		memcpy(m_szBuffer+len,sz,l);
+		memcpy(m_szBuffer+len,temp,l);
 		m_nStringLength+=l;
+		
+		if ( temp != sz )
+		{
+			free(temp);
+		}
 	}
 }
 
-#ifdef WIN32
-// original from dietlibc
-ulonglong CString::strtoull(const char *nptr, char **endptr, int base) const
+/** */
+void CString::Append( const char ch )
 {
-  longlong v=0;
-
-  while(isspace(*nptr)) ++nptr;
-
-  if (*nptr == '+') ++nptr;
-  if (!base) {
-    if (*nptr=='0') {
-      base=8;
-      if ((*(nptr+1)=='x')||(*(nptr+1)=='X')) {
-	nptr+=2;
-	base=16;
-      }
-    }
-    else
-      base=10;
-  }
-  while(*nptr) {
-    register unsigned char c=*nptr;
-    c=(c>='a'?c-'a'+10:c>='A'?c-'A'+10:c-'0');
-    if (c>=base) break;
-    v=v*base+c;
-    ++nptr;
-  }
-  if (endptr) *endptr=(char *)nptr;
-  return v;
+	if ( m_nStringLength + 2 > m_nBufferSize )
+	{
+		char * newbuf = (char*) realloc( m_szBuffer, m_nBufferSize + 200 );
+		
+		if ( newbuf == 0 )
+		{
+			perror("CString::addchar realloc ");
+			return;
+		}
+		
+		m_szBuffer = newbuf;
+		m_nBufferSize += 200;
+	}
+	
+	m_szBuffer[m_nStringLength] = ch;
+	m_szBuffer[m_nStringLength+1] = 0;
+	++m_nStringLength;
 }
-#endif
 
 /** */
 bool operator == ( const CString & string1, const CString & string2 )
 {
-	if ( string1.IsEmpty() && string2.IsEmpty() )
-		return TRUE;
+	if ( string1.IsEmpty() )
+	{
+		if ( string2.IsEmpty() )
+		{
+			return true;
+		}
+		else
+		{
+			return false;
+		}
+	}
 
 	if ( string1.Length() != string2.Length() )
-		return FALSE;
+		return false;
 
 	if ( memcmp( string1.Data(), string2.Data(), string1.Length() ) != 0 )
-		return FALSE;
+		return false;
 
-	return TRUE;
+	return true;
 }
 
 /** */
 CString CString::RightJustify( long nNewLength, char fill , bool /*truncate*/ )
 {
-	CString s="";
+	CString s;
 
 	while( (s.m_nStringLength+m_nStringLength) < nNewLength )
 		s+=fill;
@@ -810,13 +874,34 @@ CString CString::RightJustify( long nNewLength, char fill , bool /*truncate*/ )
 /** */
 bool operator == ( const char * sz, const CString & string )
 {
-	return ( CString(sz) == string );
+	if ( sz == 0 )
+	{
+		if ( string.IsEmpty() )
+		{
+			return true;
+		}
+		else
+		{
+			return false;
+		}
+	}
+	else
+	{
+		if ( strlen(sz) == (size_t) string.Length() )
+		{
+			return ( memcmp( sz, string.Data(), string.Length() ) == 0 );
+		}
+		else
+		{
+			return false;
+		}
+	}
 }
 
 /** */
 bool operator == ( const CString & string, const char * sz )
 {
-	return ( CString(sz) == string );
+	return ( sz == string );
 }
 
 /** */
@@ -828,11 +913,93 @@ bool operator != ( const CString & string1, const CString & string2 )
 /** */
 bool operator != ( const char * sz, const CString & string )
 {
-	return (!(CString(sz) == string));
+	return (!(sz == string));
 }
 
 /** */
 bool operator != ( const CString & string, const char * sz )
 {
-	return (!(CString(sz) == string));
+	return (!(sz == string));
+}
+
+/** */
+int CString::Compare( const CString & other ) const
+{
+	if ( m_nStringLength == 0 )
+	{
+		if ( other.IsEmpty() )
+		{
+			return 0;
+		}
+		else
+		{
+			return 1;
+		}
+	}
+	else if ( other.IsEmpty() )
+	{
+		return -1;
+	}
+	else
+	{
+		return strcoll( Data(), other.Data() );
+	}
+}
+
+/** */
+int CString::Compare( const char * other ) const
+{
+	if ( m_nStringLength == 0 )
+	{
+		if ( other == NULL )
+		{
+			return 0;
+		}
+		else
+		{
+			return 1;
+		}
+	}
+	else if ( other == NULL )
+	{
+		return -1;
+	}
+	else
+	{
+		return strcoll( Data(), other );
+	}
+}
+
+/** */
+bool CString::StartsWith( const CString & other ) const
+{
+	if ( other.Length() > m_nStringLength )
+	{
+		return false;
+	}
+	else if ( other.Length() == 0 )
+	{
+		return true;
+	}
+	else
+	{
+		return strncmp( m_szBuffer, other.Data(), other.Length() ) == 0;
+	}
+}
+
+/** */
+bool CString::StartsWith( const char * other, long len ) const
+{
+	if ( len > m_nStringLength )
+	{
+		return false;
+	}
+	else if ( len == 0 )
+	{
+		return true;
+	}
+	else
+	{
+		return strncmp( m_szBuffer, other, len ) == 0;
+	}
 }
