@@ -2,7 +2,7 @@
                           dcobject.h  -  description
                              -------------------
     begin                : Fri Feb 22 2002
-    copyright            : (C) 2002-2004 by Mathias Kster
+    copyright            : (C) 2002-2004 by Mathias Küster
     email                : mathen@users.berlios.de
  ***************************************************************************/
 
@@ -19,29 +19,40 @@
 #define _DCOBJECT_H_
 
 /**
-  *@author Mathias Kster
+  *@author Mathias Küster
+  *
+  * These are various dclib objects, usually there is one object
+  * for each NMDC protocol command, but some objects may be for
+  * more than one command e.g. $Get / $GetZBlock / $UGetBlock
+  * all use CMessageGet.
+  *
+  * There are also dclib objects, usually sent to valknut
+  * with information about a file or transfer.
+  *
+  * An alternative system is to have one object to handle all
+  * protocol commands, with an enum for which command and then
+  * a map of parameters and their values. Mainly for ADC,
+  * where all commands have a common format.
   */
 
 #include <time.h>
 
 #include <dclib/dcos.h>
-#include <dclib/core/cobject.h>
 #include <dclib/core/types.h>
 #include <dclib/core/cstring.h>
 #include <dclib/core/clist.h>
 #include <dclib/core/types.h>
 
+#include <list>
+
 #define DC_USER_FILELIST_HE3 	"MyList.DcLst"
 #define DC_USER_FILELIST 	DC_USER_FILELIST_HE3
 #define DC_USER_FILELIST_BZ 	"MyList.bz2"
 #define DC_USER_FILELIST_XMLBZ	"files.xml.bz2"
+#define DC_USER_FILELIST_XML	"files.xml"
 #define DC_USER_INDEXLIST	"index.lst"
 
-#ifdef WIN32
-template class DLL_EXPORT CList<CString>;
-#endif
-
-typedef enum eDCMessage {
+enum eDCMessage {
 	DC_MESSAGE_PARSE_ERROR=0,
 
 	DC_MESSAGE_CONNECTION_STATE,	// internal con. state
@@ -61,7 +72,6 @@ typedef enum eDCMessage {
 	DC_MESSAGE_CONNECTTOME,
 	DC_MESSAGE_KEY,
 	DC_MESSAGE_PRIVATECHAT,
-	DC_MESSAGE_SEARCH,
 	DC_MESSAGE_REVCONNECTTOME,
 	DC_MESSAGE_SEARCHRESULT,
 	DC_MESSAGE_FORCEMOVE,
@@ -85,7 +95,6 @@ typedef enum eDCMessage {
 	DC_MESSAGE_CANCEL,		// cancel a transfer
 	DC_MESSAGE_CANCELED,		// the transfer is canceled
 	DC_MESSAGE_SUPPORTS,
-	DC_MESSAGE_CAPABILITIES,
 
 	// download manager types
 	DC_MESSAGE_FILE_OBJECT,		// 40
@@ -117,8 +126,6 @@ typedef enum eDCMessage {
 	DC_MESSAGE_GET_NET_INFO,
 	// user command
 	DC_MESSAGE_USER_COMMAND,
-	// own ip
-	DC_MESSAGE_YOURIP,
 	// ZLIB compression
 	DC_MESSAGE_GETZBLOCK,
 	DC_MESSAGE_SENDING,
@@ -127,51 +134,39 @@ typedef enum eDCMessage {
 	DC_MESSAGE_ADCGET,
 	DC_MESSAGE_ADCSND,
 	// UserIP2 message
-	DC_MESSAGE_USERIP,
-	// ZLine message
-	DC_MESSAGE_ZLINE
+	DC_MESSAGE_USERIP
 };
 
-#define EDONKEY_HASH_BLOCK_SIZE 9728000
-
-class DLL_EXPORT CDCMessage : public CObject {
+/*
+ * The initialization phase of the constructor has to be used
+ * because m_eType is const, and it really does not make
+ * any sense for m_eType NOT to be const. Alternatively
+ * a virtual type() function could return a hard coded value.
+ */
+class CDCMessage {
 public:
 	/** */
-	CDCMessage( eDCMessage type = DC_MESSAGE_UNKNOWN ) {
-		m_eType = type;
-	};
+	CDCMessage() : m_eType( DC_MESSAGE_UNKNOWN ) {};
 	/** */
 	virtual ~CDCMessage() {};
 
 	/** */
-	CDCMessage & operator = (const CDCMessage & p)
-	{
-		m_eType = p.m_eType;
-
-		return *this;
-	};
+	const eDCMessage m_eType;
 
+protected:
 	/** */
-	eDCMessage m_eType;
+	CDCMessage( eDCMessage type ) : m_eType ( type ) {};
 };
 
-class DLL_EXPORT CMessagePing : public CDCMessage {
+class CMessagePing : public CDCMessage {
 public:
 	/** */
 	CMessagePing() : CDCMessage(DC_MESSAGE_PING) {};
 	/** */
 	virtual ~CMessagePing() {};
-
-	/** */
-	CMessagePing & operator = (const CMessagePing & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		return *this;
-	};
 };
 
-class DLL_EXPORT CMessageHubTopic : public CDCMessage {
+class CMessageHubTopic : public CDCMessage {
 public:
 	/** */
 	CMessageHubTopic() : CDCMessage(DC_MESSAGE_HUB_TOPIC) {};
@@ -181,8 +176,6 @@ public:
 	/** */
 	CMessageHubTopic & operator = (const CMessageHubTopic & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sTopic = p.m_sTopic;
 		return *this;
 	};
@@ -191,7 +184,7 @@ public:
 	CString m_sTopic;
 };
 
-class DLL_EXPORT CMessageGetInfo : public CDCMessage {
+class CMessageGetInfo : public CDCMessage {
 public:
 	/** */
 	CMessageGetInfo() : CDCMessage(DC_MESSAGE_GETINFO) {};
@@ -201,8 +194,6 @@ public:
 	/** */
 	CMessageGetInfo & operator = (const CMessageGetInfo & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sSrcNick = p.m_sSrcNick;
 		m_sDstNick = p.m_sDstNick;
 
@@ -215,23 +206,15 @@ public:
 	CString m_sDstNick;
 };
 
-class DLL_EXPORT CMessageGetNetInfo : public CDCMessage {
+class CMessageGetNetInfo : public CDCMessage {
 public:
 	/** */
 	CMessageGetNetInfo() : CDCMessage(DC_MESSAGE_GET_NET_INFO) {};
 	/** */
 	virtual ~CMessageGetNetInfo() {};
-
-	/** */
-	CMessageGetNetInfo & operator = (const CMessageGetNetInfo & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		return *this;
-	};
 };
 
-class DLL_EXPORT CMessageConnectionState : public CDCMessage {
+class CMessageConnectionState : public CDCMessage {
 public:
 	/** */
 	CMessageConnectionState() : CDCMessage(DC_MESSAGE_CONNECTION_STATE) {
@@ -243,8 +226,6 @@ public:
 	/** */
 	CMessageConnectionState & operator = (const CMessageConnectionState & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_eState   = p.m_eState;
 		m_sMessage = p.m_sMessage;
 
@@ -257,7 +238,7 @@ public:
 	CString m_sMessage;
 };
 
-class DLL_EXPORT CMessageTransfer : public CDCMessage {
+class CMessageTransfer : public CDCMessage {
 public:
 	/** */
 	CMessageTransfer() : CDCMessage(DC_MESSAGE_TRANSFER) {
@@ -271,8 +252,6 @@ public:
 	/** */
 	CMessageTransfer & operator = (const CMessageTransfer & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_eDirection  = p.m_eDirection;
 		m_nTransfered = p.m_nTransfered;
 		m_nLength     = p.m_nLength;
@@ -288,7 +267,7 @@ public:
 	ulonglong m_nLength;
 };
 
-class DLL_EXPORT CMessageFileTransferRate : public CDCMessage {
+class CMessageFileTransferRate : public CDCMessage {
 public:
 	/** */
 	CMessageFileTransferRate() : CDCMessage(DC_MESSAGE_FILE_TRANSFER_RATE) {
@@ -300,8 +279,6 @@ public:
 	/** */
 	CMessageFileTransferRate & operator = (const CMessageFileTransferRate & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sLocalFile = p.m_sLocalFile;
 		m_nRate      = p.m_nRate;
 
@@ -314,70 +291,15 @@ public:
 	ulonglong m_nRate;
 };
 
-class DLL_EXPORT CMessageSupports : public CDCMessage {
+class CMessageSupports : public CDCMessage {
 public:
 	/** */
-	CMessageSupports() : CDCMessage(DC_MESSAGE_SUPPORTS) {
-		m_bBZList    = FALSE;
-		m_bSSL       = FALSE;
-		m_bChunk     = FALSE;
-		m_bMiniSlots = FALSE;
-		m_bXMLBZList = FALSE;
-		m_bZBlock    = FALSE;
-		m_bADCGet    = FALSE;
-		m_bTTHL      = FALSE;
-		m_bTTHF      = FALSE;
-		m_bZLIG      = FALSE;
-		m_bUserCmd   = FALSE;
-		m_bNoGetInfo = FALSE;
-		m_bUserIP    = FALSE;
-		m_bUserIP2   = FALSE;
-		m_bXRules    = FALSE;
-		m_bBotINFO   = FALSE;
-		m_bNoHello   = FALSE;
-		m_bOpPlus    = FALSE;
-		m_bMCTo      = FALSE;
-		m_bBotList   = FALSE;
-		m_bClientID  = FALSE;
-		m_bSecExec   = FALSE;
-		m_bFeed      = FALSE;
-		m_bZLine     = FALSE;
-	};
+	CMessageSupports();
 	/** */
-	virtual ~CMessageSupports() {};
+	virtual ~CMessageSupports();
 
 	/** */
-	CMessageSupports & operator = (const CMessageSupports & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		m_sContent   = p.m_sContent;
-		m_bBZList    = p.m_bBZList;
-		m_bSSL       = p.m_bSSL;
-		m_bChunk     = p.m_bChunk;
-		m_bMiniSlots = p.m_bMiniSlots;
-		m_bXMLBZList = p.m_bXMLBZList;
-		m_bZBlock    = p.m_bZBlock;
-		m_bADCGet    = p.m_bADCGet;
-		m_bTTHL      = p.m_bTTHL;
-		m_bTTHF      = p.m_bTTHF;
-		m_bZLIG      = p.m_bZLIG;
-		m_bUserCmd   = p.m_bUserCmd;
-		m_bNoGetInfo = p.m_bNoGetInfo;
-		m_bUserIP    = p.m_bUserIP;
-		m_bUserIP2   = p.m_bUserIP2;
-		m_bXRules    = p.m_bXRules;
-		m_bBotINFO   = p.m_bBotINFO;
-		m_bNoHello   = p.m_bNoHello;
-		m_bOpPlus    = p.m_bOpPlus;
-		m_bMCTo      = p.m_bMCTo;
-		m_bBotList   = p.m_bBotList;
-		m_bClientID  = p.m_bClientID;
-		m_bSecExec   = p.m_bSecExec;
-		m_bFeed      = p.m_bFeed;
-		m_bZLine     = p.m_bZLine;
-		return *this;
-	};
+	CMessageSupports & operator = (const CMessageSupports & p);
 
 	/** */
 	CString m_sContent;
@@ -401,68 +323,9 @@ public:
 	bool m_bTTHF;
 	/** */
 	bool m_bZLIG;
-	/** */
-	bool m_bUserCmd;
-	/** */
-	bool m_bNoGetInfo;
-	/** */
-	bool m_bUserIP;
-	/** */
-	bool m_bUserIP2;
-	/** */
-	bool m_bXRules;
-	/** */
-	bool m_bBotINFO;
-	/** */
-	bool m_bNoHello;
-	/** */
-	bool m_bOpPlus;
-	/** */
-	bool m_bMCTo;
-	/** */
-	bool m_bBotList;
-	/** */
-	bool m_bClientID;
-	/** */
-	bool m_bSecExec;
-	/** */
-	bool m_bFeed;
-	/** */
-	bool m_bZLine;
 };
 
-class DLL_EXPORT CMessageCapabilities : public CDCMessage {
-public:
-	/** */
-	CMessageCapabilities() : CDCMessage(DC_MESSAGE_CAPABILITIES) {
-		m_bUniSearch = FALSE;
-		m_bXSearch   = FALSE;
-		m_bMD4x      = FALSE;
-	};
-	/** */
-	virtual ~CMessageCapabilities() {};
-
-	/** */
-	CMessageCapabilities & operator = (const CMessageCapabilities & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		m_bUniSearch = p.m_bUniSearch;
-		m_bXSearch   = p.m_bXSearch;
-		m_bMD4x      = p.m_bMD4x;
-
-		return *this;
-	};
-
-	/** */
-	bool m_bUniSearch;
-	/** */
-	bool m_bXSearch;
-	/** */
-	bool m_bMD4x;
-};
-
-class DLL_EXPORT CMessageError : public CDCMessage {
+class CMessageError : public CDCMessage {
 public:
 	/** */
 	CMessageError() : CDCMessage(DC_MESSAGE_ERROR) {};
@@ -472,8 +335,6 @@ public:
 	/** */
 	CMessageError & operator = (const CMessageError & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sError = p.m_sError;
 
 		return *this;
@@ -483,7 +344,7 @@ public:
 	CString m_sError;
 };
 
-class DLL_EXPORT CMessageGet : public CDCMessage {
+class CMessageGet : public CDCMessage {
 public:
 	/** */
 	CMessageGet() : CDCMessage(DC_MESSAGE_GET) {
@@ -498,8 +359,6 @@ public:
 	/** */
 	CMessageGet & operator = (const CMessageGet & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sFilename = p.m_sFilename;
 		m_nPos      = p.m_nPos;
 		m_nSize     = p.m_nSize;
@@ -521,10 +380,11 @@ public:
 	bool m_bZLib;
 };
 
-class DLL_EXPORT CMessageADCGet : public CDCMessage {
+class CMessageADCGet : public CDCMessage {
 public:
 	/** */
 	CMessageADCGet() : CDCMessage(DC_MESSAGE_ADCGET) {
+		m_eADCType = eAdcFile;
 		m_nPos  = 0;
 		m_nSize = -1;
 		m_bZlib = false;
@@ -535,20 +395,19 @@ public:
 	/** */
 	CMessageADCGet & operator = (const CMessageADCGet & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-		
-		m_sType = p.m_sType;
+		m_eADCType = p.m_eADCType;
 		m_sTTH  = p.m_sTTH;
 		m_nPos  = p.m_nPos;
 		m_nSize = p.m_nSize;
 		m_bZlib = p.m_bZlib;
+		m_sFile = p.m_sFile;
 		
 		return *this;
 	};
 	
-	/** file, tthl, or list (only file supported) */
-	CString m_sType;
-	/** */
+	/** file, tthl, or list */
+	eADCType m_eADCType;
+	/** TTH without prefix or empty if using file */
 	CString m_sTTH;
 	/** */
 	ulonglong m_nPos;
@@ -556,12 +415,15 @@ public:
 	long long m_nSize;
 	/** */
 	bool m_bZlib;
+	/** File if not using TTH, empty otherwise */
+	CString m_sFile;
 };
 
-class DLL_EXPORT CMessageADCSnd : public CDCMessage {
+class CMessageADCSnd : public CDCMessage {
 public:
 	/** */
 	CMessageADCSnd() : CDCMessage(DC_MESSAGE_ADCSND) {
+		m_eADCType = eAdcFile;
 		m_nPos  = 0;
 		m_nSize = -1;
 		m_bZlib = false;
@@ -572,20 +434,19 @@ public:
 	/** */
 	CMessageADCSnd & operator = (const CMessageADCSnd & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-		
-		m_sType = p.m_sType;
+		m_eADCType = p.m_eADCType;
 		m_sTTH  = p.m_sTTH;
 		m_nPos  = p.m_nPos;
 		m_nSize = p.m_nSize;
 		m_bZlib = p.m_bZlib;
+		m_sFile = p.m_sFile;
 		
 		return *this;
 	};
 	
-	/** file, tthl, or list (only file supported) */
-	CString m_sType;
-	/** */
+	/** file, tthl, or list */
+	eADCType m_eADCType;
+	/** TTH without prefix or empty if using file*/
 	CString m_sTTH;
 	/** */
 	ulonglong m_nPos;
@@ -593,89 +454,51 @@ public:
 	long long m_nSize;
 	/** */
 	bool m_bZlib;
+	/** File if not using TTH, empty otherwise */
+	CString m_sFile;
 };
 
-class DLL_EXPORT CMessageGetPass : public CDCMessage {
+class CMessageGetPass : public CDCMessage {
 public:
 	/** */
 	CMessageGetPass() : CDCMessage(DC_MESSAGE_GETPASS) {};
 	/** */
 	virtual ~CMessageGetPass() {};
-
-	/** */
-	CMessageGetPass & operator = (const CMessageGetPass & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		return *this;
-	};
 };
 
-class DLL_EXPORT CMessageCancel : public CDCMessage {
+class CMessageCancel : public CDCMessage {
 public:
 	/** */
 	CMessageCancel() : CDCMessage(DC_MESSAGE_CANCEL) {};
 	/** */
 	virtual ~CMessageCancel() {};
-
-	/** */
-	CMessageCancel & operator = (const CMessageCancel & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		return *this;
-	};
 };
 
-class DLL_EXPORT CMessageCanceled : public CDCMessage {
+class CMessageCanceled : public CDCMessage {
 public:
 	/** */
 	CMessageCanceled() : CDCMessage(DC_MESSAGE_CANCELED) {};
 	/** */
 	virtual ~CMessageCanceled() {};
-
-	/** */
-	CMessageCanceled & operator = (const CMessageCanceled & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		return *this;
-	};
 };
 
-class DLL_EXPORT CMessageKey : public CDCMessage {
+class CMessageKey : public CDCMessage {
 public:
 	/** */
 	CMessageKey() : CDCMessage(DC_MESSAGE_KEY) {};
 	/** */
 	virtual ~CMessageKey() {};
-
-	/** */
-	CMessageKey & operator = (const CMessageKey & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		return *this;
-	};
 };
 
-class DLL_EXPORT CMessageBadPass : public CDCMessage {
+class CMessageBadPass : public CDCMessage {
 public:
 	/** */
 	CMessageBadPass() : CDCMessage(DC_MESSAGE_BADPASS) {};
 	/** */
 	virtual ~CMessageBadPass() {};
-
-	/** */
-	CMessageBadPass & operator = (const CMessageBadPass & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		return *this;
-	};
 };
 
-class DLL_EXPORT CMessageLogedIn : public CDCMessage {
+class CMessageLogedIn : public CDCMessage {
 public:
 	/** */
 	CMessageLogedIn() : CDCMessage(DC_MESSAGE_LOGEDIN) {};
@@ -685,8 +508,6 @@ public:
 	/** */
 	CMessageLogedIn & operator = (const CMessageLogedIn & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sNick = p.m_sNick;
 
 		return *this;
@@ -696,92 +517,52 @@ public:
 	CString m_sNick;
 };
 
-class DLL_EXPORT CMessageSend : public CDCMessage {
+class CMessageSend : public CDCMessage {
 public:
 	/** */
 	CMessageSend() : CDCMessage(DC_MESSAGE_SEND) {};
 	/** */
 	virtual ~CMessageSend() {};
-
-	/** */
-	CMessageSend & operator = (const CMessageSend & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		return *this;
-	};
 };
 
-class DLL_EXPORT CMessageValidateDenide : public CDCMessage {
+class CMessageValidateDenide : public CDCMessage {
 public:
 	/** */
 	CMessageValidateDenide() : CDCMessage(DC_MESSAGE_VALIDATEDENIDE) {};
 	/** */
 	virtual ~CMessageValidateDenide() {};
-
-	/** */
-	CMessageValidateDenide & operator = (const CMessageValidateDenide & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		return *this;
-	};
 };
 
-class DLL_EXPORT CMessageHubIsFull : public CDCMessage {
+class CMessageHubIsFull : public CDCMessage {
 public:
 	/** */
 	CMessageHubIsFull() : CDCMessage(DC_MESSAGE_HUBISFULL) {};
 	/** */
 	virtual ~CMessageHubIsFull() {};
-
-	/** */
-	CMessageHubIsFull & operator = (const CMessageHubIsFull & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		return *this;
-	};
 };
 
-class DLL_EXPORT CMessageMaxedOut : public CDCMessage {
+class CMessageMaxedOut : public CDCMessage {
 public:
 	/** */
 	CMessageMaxedOut() : CDCMessage(DC_MESSAGE_MAXEDOUT) {};
 	/** */
 	virtual ~CMessageMaxedOut() {};
-
-	/** */
-	CMessageMaxedOut & operator = (const CMessageMaxedOut & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		return *this;
-	};
 };
 
-class DLL_EXPORT CMessageGetListLen : public CDCMessage {
+class CMessageGetListLen : public CDCMessage {
 public:
 	/** */
 	CMessageGetListLen() : CDCMessage(DC_MESSAGE_GETLISTLEN) {};
 	/** */
 	virtual ~CMessageGetListLen() {};
-
-	/** */
-	CMessageGetListLen & operator = (const CMessageGetListLen & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		return *this;
-	};
 };
 
-class DLL_EXPORT CMessageLock : public CDCMessage {
+class CMessageLock : public CDCMessage {
 public:
 	/** */
 	CMessageLock() : CDCMessage(DC_MESSAGE_LOCK) {
 		m_eClientVersion = eucvNONE;
-		m_bExtProtocol   = FALSE;
+		m_bExtProtocol   = false;
 		m_nVersionMajor  = 0;
 		m_nVersionMinor  = 0;
 		m_nVersionPatch  = 0;
@@ -792,8 +573,6 @@ public:
 	/** */
 	CMessageLock & operator = (const CMessageLock & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sPK            = p.m_sPK;
 		m_sData          = p.m_sData;
 		m_eClientVersion = p.m_eClientVersion;
@@ -824,7 +603,7 @@ public:
 	bool m_bExtProtocol;
 };
 
-class DLL_EXPORT CMessageFileLength : public CDCMessage {
+class CMessageFileLength : public CDCMessage {
 public:
 	/** */
 	CMessageFileLength() : CDCMessage(DC_MESSAGE_FILELENGTH) {
@@ -836,8 +615,6 @@ public:
 	/** */
 	CMessageFileLength & operator = (const CMessageFileLength & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_nFileLength = p.m_nFileLength;
 
 		return *this;
@@ -847,7 +624,7 @@ public:
 	ulonglong m_nFileLength;
 };
 
-class DLL_EXPORT CMessageSending : public CDCMessage {
+class CMessageSending : public CDCMessage {
 public:
 	/** */
 	CMessageSending() : CDCMessage(DC_MESSAGE_SENDING) {
@@ -859,8 +636,6 @@ public:
 	/** */
 	CMessageSending & operator = (const CMessageSending & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_nLength = p.m_nLength;
 
 		return *this;
@@ -870,7 +645,7 @@ public:
 	ulonglong m_nLength;
 };
 
-class DLL_EXPORT CMessageDirection : public CDCMessage {
+class CMessageDirection : public CDCMessage {
 public:
 	/** */
 	CMessageDirection() : CDCMessage(DC_MESSAGE_DIRECTION) {
@@ -883,8 +658,6 @@ public:
 	/** */
 	CMessageDirection & operator = (const CMessageDirection & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_eDirection = p.m_eDirection;
 		m_nLevel     = p.m_nLevel;
 
@@ -897,47 +670,43 @@ public:
 	int m_nLevel;
 };
 
-class DLL_EXPORT CMessageNickList : public CDCMessage {
+class CMessageNickList : public CDCMessage {
 public:
 	/** */
-	CMessageNickList() : CDCMessage(DC_MESSAGE_NICKLIST) {};
+	CMessageNickList();
+	/**
+	 * Copy constructor that does a deep copy of the list.
+	 */
+	CMessageNickList( const CMessageNickList & other );
 	/** */
-	virtual ~CMessageNickList() {};
-
-	CMessageNickList & operator = (const CMessageNickList & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		// TODO: copy list
-
-		return *this;
-	};
+	virtual ~CMessageNickList();
+	
+	/** */
+	CMessageNickList & operator = (const CMessageNickList & rhs);
 
 	/** */
 	CList<CString> m_NickList;
 };
 
-class DLL_EXPORT CMessageNick : public CDCMessage {
+class CMessageOpList : public CDCMessage {
 public:
 	/** */
-	CMessageNick() {};
+	CMessageOpList();
+	/**
+	 * Copy constructor that does a deep copy of the list.
+	 */
+	CMessageOpList( const CMessageOpList & other );
 	/** */
-	virtual ~CMessageNick() {};
-
-	CMessageNick & operator = (const CMessageNick & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		m_sNick = p.m_sNick;
-
-		return *this;
-	};
+	virtual ~CMessageOpList();
+	
+	/** */
+	CMessageOpList & operator = (const CMessageOpList & rhs);
 
 	/** */
-	CString m_sNick;
+	CList<CString> m_NickList;
 };
 
-class DLL_EXPORT CMessageMyNick : public CDCMessage {
+class CMessageMyNick : public CDCMessage {
 public:
 	/** */
 	CMessageMyNick() : CDCMessage(DC_MESSAGE_MYNICK) {};
@@ -946,8 +715,6 @@ public:
 
 	CMessageMyNick & operator = (const CMessageMyNick & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sNick = p.m_sNick;
 
 		return *this;
@@ -957,7 +724,7 @@ public:
 	CString m_sNick;
 };
 
-class DLL_EXPORT CMessageHello : public CDCMessage {
+class CMessageHello : public CDCMessage {
 public:
 	/** */
 	CMessageHello() : CDCMessage(DC_MESSAGE_HELLO) {};
@@ -966,8 +733,6 @@ public:
 
 	CMessageHello & operator = (const CMessageHello & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sNick = p.m_sNick;
 
 		return *this;
@@ -977,7 +742,7 @@ public:
 	CString m_sNick;
 };
 
-class DLL_EXPORT CMessageQuit : public CDCMessage {
+class CMessageQuit : public CDCMessage {
 public:
 	/** */
 	CMessageQuit() : CDCMessage(DC_MESSAGE_QUIT) {};
@@ -986,8 +751,6 @@ public:
 
 	CMessageQuit & operator = (const CMessageQuit & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sNick = p.m_sNick;
 
 		return *this;
@@ -997,7 +760,7 @@ public:
 	CString m_sNick;
 };
 
-class DLL_EXPORT CMessageChat : public CDCMessage {
+class CMessageChat : public CDCMessage {
 public:
 	/** */
 	CMessageChat() : CDCMessage(DC_MESSAGE_CHAT) {};
@@ -1006,8 +769,6 @@ public:
 
 	CMessageChat & operator = (const CMessageChat & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sNick    = p.m_sNick;
 		m_sMessage = p.m_sMessage;
 
@@ -1020,104 +781,78 @@ public:
 	CString m_sMessage;
 };
 
-class DLL_EXPORT CMessageMyInfo : public CDCMessage {
+class CMessageMyInfo : public CDCMessage {
 public:
 	/** */
-	CMessageMyInfo() : CDCMessage(DC_MESSAGE_MYINFO) {
-		m_bValid         = FALSE;
-		m_nShared        = 0;
-		m_bOperator      = FALSE;
-		m_eAwayMode      = euamNONE;
-		m_eClientVersion = eucvNONE;
-		m_nFreeSlot      = 0;
-		m_nMaxSlot       = 0;
-		m_eUserSpeed     = eusUNKNOWN;
-		m_eClientMode    = ecmNONE;
-	};
+	CMessageMyInfo();
 	/** */
-	virtual ~CMessageMyInfo() {};
+	virtual ~CMessageMyInfo();
 
 	/** */
-	CMessageMyInfo & operator = (const CMessageMyInfo & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
+	CMessageMyInfo & operator = (const CMessageMyInfo & p);
 
-		m_sNick          = p.m_sNick;
-		m_bValid         = p.m_bValid;
-		m_sComment       = p.m_sComment;
-		m_sUnknown       = p.m_sUnknown;
-		m_eUserSpeed     = p.m_eUserSpeed;
-		m_sUserSpeed     = p.m_sUserSpeed;
-		m_sEMail         = p.m_sEMail;
-		m_nShared        = p.m_nShared;
-		m_eAwayMode      = p.m_eAwayMode;
-		m_bOperator      = p.m_bOperator;
-		m_sVerComment    = p.m_sVerComment;
-		m_eClientVersion = p.m_eClientVersion;
-		m_eClientMode    = p.m_eClientMode;
-		m_nFreeSlot      = p.m_nFreeSlot;
-		m_nMaxSlot       = p.m_nMaxSlot;
-		m_sTransferHost  = p.m_sTransferHost;
-		m_MessageLock    = p.m_MessageLock;
-		m_MessageSupports = p.m_MessageSupports;
-
-		return *this;
-	};
-
-	/** the nickname */
-	CString m_sNick;
-	/** true if myinfo recv. */
-	bool m_bValid;
-	/** the comment */
-	CString m_sComment;
-	/** */
-	CString m_sUnknown;
-	/** parsed speed */
-	eUserSpeed m_eUserSpeed;
-	/** the speed string */
-	CString m_sUserSpeed;
-	/** the email */
-	CString m_sEMail;
-	/** sharesize */
-	ulonglong m_nShared;
-	/** away mode */
-	eUserAwayMode m_eAwayMode;
-	/** operator status */
-	bool m_bOperator;
-	/** version */
-	CString m_sVerComment;
-	/** parsed client version */
-	eUserClientVersion m_eClientVersion;
-	/** client mode */
-	eClientMode m_eClientMode;
-	/** free slots */
-	unsigned int m_nFreeSlot;
-	/** max slots */
-	unsigned int m_nMaxSlot;
-	/** */
-	CString m_sTransferHost;
 	/** */
 	CMessageLock m_MessageLock;
 	/** */
 	CMessageSupports m_MessageSupports;
+
+	/** the nickname */
+	CString m_sNick;
+	/** the comment */
+	CString m_sComment;
+	/** */
+	CString m_sUnknown;
+	/** the speed string */
+	CString m_sUserSpeed;
+	/** the email */
+	CString m_sEMail;
+	/** version */
+	CString m_sVerComment;
+	/** */
+	CString m_sTransferHost;
+
+	/** sharesize */
+	ulonglong m_nShared;
+	/** parsed speed */
+	eUserSpeed m_eUserSpeed;
+	/** away mode */
+	eUserAwayMode m_eAwayMode;
+	/** parsed client version */
+	eUserClientVersion m_eClientVersion;
+	/** client mode */
+	eClientMode m_eClientMode;
+	/** true if myinfo recv. */
+	bool m_bValid;
+	/** operator status */
+	bool m_bOperator;
+	/** Server flag set - special icon (not used) */
+	bool m_bServerFlag;
+	/** Fireball flag set - special icon (not used) */
+	bool m_bFireballFlag;
+	/** Supports NMDC TLS port */
+	bool m_bTLSFlag;
+	
+protected:
+	/** For CMessageSearchResultUser */
+	CMessageMyInfo( eDCMessage type );
 };
 
-class DLL_EXPORT CMessageConnectToMe : public CDCMessage {
+class CMessageConnectToMe : public CDCMessage {
 public:
 	/** */
 	CMessageConnectToMe() : CDCMessage(DC_MESSAGE_CONNECTTOME) {
 		m_nPort = 0;
+		m_bCrypto = false;
 	};
 	/** */
 	virtual ~CMessageConnectToMe() {};
 
 	CMessageConnectToMe & operator = (const CMessageConnectToMe & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sNick = p.m_sNick;
 		m_sHost = p.m_sHost;
 		m_nPort = p.m_nPort;
+		m_bCrypto = p.m_bCrypto;
 
 		return *this;
 	};
@@ -1128,9 +863,11 @@ public:
 	CString m_sHost;
 	/** */
 	int m_nPort;
+	/** StrongDC NMDC TLS port extension */
+	bool m_bCrypto;
 };
 
-class DLL_EXPORT CMessageRevConnectToMe : public CDCMessage {
+class CMessageRevConnectToMe : public CDCMessage {
 public:
 	/** */
 	CMessageRevConnectToMe() : CDCMessage(DC_MESSAGE_REVCONNECTTOME) {};
@@ -1139,8 +876,6 @@ public:
 
 	CMessageRevConnectToMe & operator = (const CMessageRevConnectToMe & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sNick    = p.m_sNick;
 		m_sDstNick = p.m_sDstNick;
 
@@ -1153,60 +888,7 @@ public:
 	CString m_sDstNick;
 };
 
-class DLL_EXPORT CMessageSearch : public CDCMessage {
-public:
-	/** */
-	CMessageSearch() : CDCMessage(DC_MESSAGE_SEARCH) {
-		m_bLocal      = FALSE;
-		m_nPort       = 0;
-		m_bSizeLimit  = FALSE;
-		m_eSizeType   = esstATLEAST;
-		m_nSize       = 0;
-		m_eFileType   = eftUNKNOWN;
-		m_bExtended   = FALSE;
-	};
-	/** */
-	virtual ~CMessageSearch() {};
-
-	/** */
-	CMessageSearch & operator = (const CMessageSearch & p)
-	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
-		m_bLocal      = p.m_bLocal;
-		m_sSource     = p.m_sSource;
-		m_nPort       = p.m_nPort;
-		m_bSizeLimit  = p.m_bSizeLimit;
-		m_eSizeType   = p.m_eSizeType;
-		m_nSize       = p.m_nSize;
-		m_eFileType   = p.m_eFileType;
-		m_sString     = p.m_sString;
-		m_bExtended   = p.m_bExtended;
-
-		return *this;
-	};
-
-	/** local or global search */
-	bool m_bLocal;
-	/** the source, nick or host */
-	CString m_sSource;
-	/** if source a host, this is the port */
-	int m_nPort;
-	/** search options */
-	bool m_bSizeLimit;
-	/** search options */
-	enum eSearchSizeType m_eSizeType;
-	/** search options */
-	ulonglong m_nSize;
-	/** search options */
-	eFileTypes m_eFileType;
-	/** search string */
-	CString m_sString;
-	/** extended protocol */
-	bool m_bExtended;
-};
-
-class DLL_EXPORT CMessageForceMove : public CDCMessage {
+class CMessageForceMove : public CDCMessage {
 public:
 	/** */
 	CMessageForceMove() : CDCMessage(DC_MESSAGE_FORCEMOVE) {
@@ -1218,8 +900,6 @@ public:
 	/** */
 	CMessageForceMove & operator = (const CMessageForceMove & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sHost = p.m_sHost;
 		m_nPort = p.m_nPort;
 
@@ -1232,13 +912,14 @@ public:
 	int m_nPort;
 };
 
-class DLL_EXPORT CMessageSearchResult : public CDCMessage {
+class CMessageSearchResult : public CDCMessage {
 public:
 	/** */
 	CMessageSearchResult() : CDCMessage(DC_MESSAGE_SEARCHRESULT) {
 		m_nFreeSlot = 0;
 		m_nMaxSlot  = 0;
 		m_nSize     = 0;
+		m_bFolder   = false;
 	};
 	/** */
 	virtual ~CMessageSearchResult() {};
@@ -1246,14 +927,13 @@ public:
 	/** */
 	CMessageSearchResult & operator = (const CMessageSearchResult & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sNick     = p.m_sNick;
 		m_sFile     = p.m_sFile;
 		m_nSize     = p.m_nSize;
 		m_sHubName  = p.m_sHubName;
 		m_sHubHost  = p.m_sHubHost;
 		m_sHash     = p.m_sHash;
+		m_sSrcIP    = p.m_sSrcIP;
 		m_nFreeSlot = p.m_nFreeSlot;
 		m_nMaxSlot  = p.m_nMaxSlot;
 		m_bFolder   = p.m_bFolder;
@@ -1273,6 +953,11 @@ public:
 	CString m_sHubHost;
 	/** */
 	CString m_sHash;
+	/**
+	 * The source (client not hub) IP if you are in active mode,
+	 * always empty if you are in passive mode.
+	 */
+	CString m_sSrcIP;
 	/** */
 	unsigned int m_nFreeSlot;
 	/** */
@@ -1281,7 +966,7 @@ public:
 	bool m_bFolder;
 };
 
-class DLL_EXPORT CMessagePrivateChat : public CDCMessage {
+class CMessagePrivateChat : public CDCMessage {
 public:
 	/** */
 	CMessagePrivateChat() : CDCMessage(DC_MESSAGE_PRIVATECHAT) {
@@ -1293,8 +978,6 @@ public:
 	/** */
 	CMessagePrivateChat & operator = (const CMessagePrivateChat & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sDstNick      = p.m_sDstNick;
 		m_sSrcNick      = p.m_sSrcNick;
 		m_sMultiSrcNick = p.m_sMultiSrcNick;
@@ -1316,7 +999,7 @@ public:
 	eSecureState m_eSecureState;
 };
 
-class DLL_EXPORT CMessageHubName : public CDCMessage {
+class CMessageHubName : public CDCMessage {
 public:
 	/** */
 	CMessageHubName() : CDCMessage(DC_MESSAGE_HUBNAME) {};
@@ -1326,8 +1009,6 @@ public:
 	/** */
 	CMessageHubName & operator = (const CMessageHubName & p)
 	{
-		(CDCMessage&)*this = (CDCMessage&)p;
-
 		m_sHubName    = p.m_sHubName;
 		m_sOldHubName = p.m_sOldHubName;
 
@@ -1342,47 +1023,17 @@ public:
 
 // download manager object classes
 
-class DLL_EXPORT CMessageDMFileObject : public CDCMessage {
+class CMessageDMFileObject : public CDCMessage {
 public:
 	/** */
-	CMessageDMFileObject() : CDCMessage(DC_MESSAGE_FILE_OBJECT) {
-		m_tTimeout           = 0;
-		m_eTransferWaitState = etwsWAIT;
-		m_nConnections       = 0;
-		m_eTransferFileState = etfsNONE;
-		m_nSize              = 0;
-		m_nStartPosition     = 0;
-		m_nCurrentPosition   = 0;
-		m_nEndPosition       = 0;
-		m_bMulti             = FALSE;
-		m_bRemoveFile        = FALSE;
-		m_nPriority          = 0;
-	};
+	CMessageDMFileObject();
+	/** Copy constructor that does deep copy */
+	CMessageDMFileObject( const CMessageDMFileObject & other );
 	/** */
-	virtual ~CMessageDMFileObject() {};
+	virtual ~CMessageDMFileObject();
 
 	/** */
-	CMessageDMFileObject & operator = (const CMessageDMFileObject & p)
-	{
-		m_sNick              = p.m_sNick;
-		m_sHubName           = p.m_sHubName;
-		m_sHubHost           = p.m_sHubHost;
-		m_tTimeout           = p.m_tTimeout;
-		m_eTransferWaitState = p.m_eTransferWaitState;
-		m_nConnections       = p.m_nConnections;
-		m_sRemoteFile        = p.m_sRemoteFile;
-		m_sLocalFile         = p.m_sLocalFile;
-		m_eTransferFileState = p.m_eTransferFileState;
-		m_nSize              = p.m_nSize;
-		m_nStartPosition     = p.m_nStartPosition;
-		m_nCurrentPosition   = p.m_nCurrentPosition;
-		m_nEndPosition       = p.m_nEndPosition;
-		m_bMulti             = p.m_bMulti;
-		m_bRemoveFile        = p.m_bRemoveFile;
-		m_nPriority          = p.m_nPriority;
-
-		return *this;
-	};
+	CMessageDMFileObject & operator = (const CMessageDMFileObject & p);
 
 	/** */
 	CString m_sNick;
@@ -1390,16 +1041,18 @@ public:
 	CString m_sHubName;
 	/** */
 	CString m_sHubHost;
+	/** remote file */
+	CString m_sRemoteFile;
+	/** the local file */
+	CString m_sLocalFile;
+	/** */
+	CString m_sTTH;
 	/** */
 	time_t m_tTimeout;
 	/** */
 	eTransferWaitState m_eTransferWaitState;
 	/** */
 	int m_nConnections;
-	/** remote file */
-	CString m_sRemoteFile;
-	/** the local file */
-	CString m_sLocalFile;
 	/** */
 	eTransferFileState m_eTransferFileState;
 	/** file size */
@@ -1411,55 +1064,24 @@ public:
 	/** file end position */
 	ulonglong m_nEndPosition;
 	/** */
+	int m_nPriority;
+	/** list of directories to download */
+	std::list<CString> * m_pDirList;
+	/** */
 	bool m_bMulti;
 	/** */
 	bool m_bRemoveFile;
-	/** */
-	int m_nPriority;
 };
 
-class DLL_EXPORT CMessageDMTransferObject : public CDCMessage {
+class CMessageDMTransferObject : public CDCMessage {
 public:
 	/** */
-	CMessageDMTransferObject() : CDCMessage(DC_MESSAGE_TRANSFER_OBJECT) {
-		m_nTransferID    = 0;
-		lSize            = 0;
-		lSizeDone        = 0;
-		lRate            = 0;
-		lStartPosition   = 0;
-		lTransfered      = 0;
-		lEndPosition     = 0;
-		eState           = estNONE;
-		bRemoveTransfer  = FALSE;
-		m_nMultiRate     = 0;
-		m_bEncrypted     = FALSE;
-	};
+	CMessageDMTransferObject();
 	/** */
-	virtual ~CMessageDMTransferObject() {};
+	virtual ~CMessageDMTransferObject();
 
 	/** */
-	CMessageDMTransferObject & operator = (const CMessageDMTransferObject & p)
-	{
-		m_nTransferID   = p.m_nTransferID;
-		m_sSrcNick      = p.m_sSrcNick;
-		m_sDstNick      = p.m_sDstNick;
-		sHost           = p.sHost;
-		m_sHubHost      = p.m_sHubHost;
-		sHubName        = p.sHubName;
-		m_sDstFile      = p.m_sDstFile;
-		m_sSrcFile      = p.m_sSrcFile;
-		lSize           = p.lSize;
-		lSizeDone       = p.lSizeDone;
-		lRate           = p.lRate;
-		m_nMultiRate    = p.m_nMultiRate;
-		lStartPosition  = p.lStartPosition;
-		lTransfered     = p.lTransfered;
-		lEndPosition    = p.lEndPosition;
-		bRemoveTransfer = p.bRemoveTransfer;
-		m_bEncrypted    = p.m_bEncrypted;
-
-		return *this;
-	};
+	CMessageDMTransferObject & operator = (const CMessageDMTransferObject & p);
 
 	/** */
 	ulonglong m_nTransferID;
@@ -1473,13 +1095,14 @@ public:
 	CString m_sHubHost;
 	/** */
 	CString sHubName;
-	/** */
-	eConnectionState eState;
-
 	/** remote file */
 	CString m_sDstFile;
 	/** local file */
 	CString m_sSrcFile;
+	/** */
+	CString m_sTTH;
+	/** */
+	eConnectionState eState;
 	/** file size */
 	ulonglong lSize;
 	/** file size done */
@@ -1500,23 +1123,17 @@ public:
 	bool m_bEncrypted;
 };
 
-class DLL_EXPORT CMessageDMFileListObject : public CDCMessage {
+class CMessageDMFileListObject : public CDCMessage {
 public:
 	/** */
-	CMessageDMFileListObject() : CDCMessage(DC_MESSAGE_FILELIST_OBJECT) {};
+	CMessageDMFileListObject();
+	/** Does a deep copy of the optional list */
+	CMessageDMFileListObject( const CMessageDMFileListObject & other );
 	/** */
-	virtual ~CMessageDMFileListObject() {};
+	virtual ~CMessageDMFileListObject();
 
 	/** */
-	CMessageDMFileListObject & operator = (const CMessageDMFileListObject & p)
-	{
-		sNick      = p.sNick;
-		sHubName   = p.sHubName;
-		sHubHost   = p.sHubHost;
-		sLocalFile = p.sLocalFile;
-
-		return *this;
-	};
+	CMessageDMFileListObject & operator = (const CMessageDMFileListObject & p);
 
 	/** */
 	CString sNick;
@@ -1526,14 +1143,18 @@ public:
 	CString sHubHost;
 	/** */
 	CString sLocalFile;
+	/** open filelist at file/folder */
+	CString sJumpTo;
+	/** list of directories to download */
+	std::list<CString> * m_pDirList;
 };
 
-class DLL_EXPORT CMessageDMSlotObject : public CDCMessage {
+class CMessageDMSlotObject : public CDCMessage {
 public:
 	/** */
 	CMessageDMSlotObject() : CDCMessage(DC_MESSAGE_SLOT_OBJECT) {
 		iSlots     = 0;
-		bPermanent = FALSE;
+		bPermanent = false;
 	};
 	/** */
 	virtual ~CMessageDMSlotObject() {};
@@ -1548,7 +1169,7 @@ public:
 	bool bPermanent;
 };
 
-class DLL_EXPORT CMessageLog : public CDCMessage {
+class CMessageLog : public CDCMessage {
 public:
 	/** */
 	CMessageLog() : CDCMessage(DC_MESSAGE_LOG) {};
@@ -1559,58 +1180,83 @@ public:
 	CString sMessage;
 };
 
-class DLL_EXPORT CMessageSearchUser : public CMessageNick {
+class CMessageSearchUser : public CDCMessage {
 public:
 	/** */
-	CMessageSearchUser() {
-		m_eType = DC_MESSAGE_SEARCH_USER;
-	};
+	CMessageSearchUser() : CDCMessage(DC_MESSAGE_SEARCH_USER) {};
 	/** */
 	virtual ~CMessageSearchUser() {};
+	
+	/** */
+	CString m_sNick;
 };
 
-class DLL_EXPORT CMessageSearchResultUser : public CMessageMyInfo {
+class CMessageSearchResultUser : public CMessageMyInfo {
 public:
 	/** */
-	CMessageSearchResultUser() {
-		m_eType = DC_MESSAGE_SEARCHRESULT_USER;
-	};
+	CMessageSearchResultUser() : CMessageMyInfo( DC_MESSAGE_SEARCHRESULT_USER ) {};
 	/** */
 	virtual ~CMessageSearchResultUser() {};
 
+	CMessageSearchResultUser & operator = (const CMessageSearchResultUser & rhs)
+	{
+		(CMessageMyInfo&)*this = (CMessageMyInfo&)rhs;
+		
+		m_sHubName = rhs.m_sHubName;
+		
+		return *this;
+	};
+
 	/** */
 	CString m_sHubName;
 };
 
-class DLL_EXPORT CMessageSearchFile : public CMessageSearch {
+class CMessageSearchFile : public CDCMessage {
 public:
 	/** */
-	CMessageSearchFile() {
-		m_eType  = DC_MESSAGE_SEARCH_FILE;
-		m_bMulti = FALSE;
-	};
+	CMessageSearchFile();
 	/** */
-	virtual ~CMessageSearchFile() {};
+	virtual ~CMessageSearchFile();
 
+	CMessageSearchFile & operator = (const CMessageSearchFile & rhs);
+
+	/** search string */
+	CString m_sString;
+	/** the source, nick or host */
+	CString m_sSource;
+	/** if source a host, this is the port */
+	int m_nPort;
+	/** search options */
+	enum eSearchSizeType m_eSizeType;
+	/** search options */
+	ulonglong m_nSize;
+	/** search options */
+	eFileTypes m_eFileType;
+	/** search options */
+	bool m_bSizeLimit;
+	/** local or global search */
+	bool m_bLocal;
+	/** extended protocol */
+	bool m_bExtended;
 	/** hub network search */
 	bool m_bMulti;
 };
 
-class DLL_EXPORT DCHubObject : public CObject {
+class DCHubObject {
 public:
 	/** */
 	DCHubObject() {
-		m_bActive = FALSE;
-	}
+		m_bActive = false;
+	};
 	/** */
 	DCHubObject( DCHubObject * o ) {
 		m_bActive  = o->m_bActive;
 		m_sHubName = o->m_sHubName;
 		m_sHubHost = o->m_sHubHost;
-	}
+	};
 
 	/** */
-	virtual ~DCHubObject() {};
+	~DCHubObject() {};
 
 	/** */
 	bool m_bActive;
@@ -1620,7 +1266,7 @@ public:
 	CString m_sHubHost;
 };
 
-class DLL_EXPORT DCMessageTraffic : public CDCMessage {
+class DCMessageTraffic : public CDCMessage {
 public:
 	/** */
 	DCMessageTraffic() : CDCMessage(DC_MESSAGE_TRAFFIC) {
@@ -1630,7 +1276,7 @@ public:
 		m_nDataTx    = 0;
 		m_nControlRx = 0;
 		m_nControlTx = 0;
-	}
+	};
 	/** */
 	virtual ~DCMessageTraffic() {};
 
@@ -1660,12 +1306,12 @@ public:
 	ulonglong m_nControlTx;
 };
 
-class DLL_EXPORT DCMessageGetHubList : public CDCMessage {
+class DCMessageGetHubList : public CDCMessage {
 public:
 	/** */
 	DCMessageGetHubList() : CDCMessage(DC_MESSAGE_GETHUBLIST) {
-		m_bRun = FALSE;
-	}
+		m_bRun = false;
+	};
 	/** */
 	virtual ~DCMessageGetHubList() {};
 
@@ -1673,11 +1319,10 @@ public:
 	bool m_bRun;
 };
 
-class DLL_EXPORT DCMessageConnectClient : public CDCMessage {
+class DCMessageConnectClient : public CDCMessage {
 public:
 	/** */
-	DCMessageConnectClient() : CDCMessage(DC_MESSAGE_CONNECT_CLIENT) {
-	}
+	DCMessageConnectClient() : CDCMessage(DC_MESSAGE_CONNECT_CLIENT) {};
 	/** */
 	virtual ~DCMessageConnectClient() {};
 
@@ -1687,27 +1332,37 @@ public:
 	CString m_sHubHost;
 };
 
-class DLL_EXPORT CMessageUserIP : public CDCMessage {
+class CMessageUserIP : public CDCMessage {
 public:
 	/** Creates a new UserIP object */
-	CMessageUserIP() : CDCMessage(DC_MESSAGE_USERIP) {
-	}
+	CMessageUserIP() : CDCMessage(DC_MESSAGE_USERIP) {};
 	/** */
 	virtual ~CMessageUserIP() {};
 	
-	/** Nick */
-	CString m_sNick;
-	/** IP address */
-	CString m_sIP;
+	/** List of nicks */
+	std::list<CString> m_lNicks;
+	/** List of IPs */
+	std::list<CString> m_lIPs;
 };
 
-class DLL_EXPORT CMessageZLine : public CDCMessage {
+class CMessageUserCommand : public CDCMessage {
 public:
 	/** */
-	CMessageZLine() : CDCMessage(DC_MESSAGE_ZLINE) {
-	}
+	CMessageUserCommand() : CDCMessage(DC_MESSAGE_USER_COMMAND) {
+		type = -1;
+		context = -1;
+	};
 	/** */
-	virtual ~CMessageZLine() {};
+	virtual ~CMessageUserCommand() {};
+	
+	/** Command type - raw, raw once, remove or clear commands etc. */
+	int type;
+	/** Command context - which menus it goes in */
+	int context;
+	/** Command name */
+	CString name;
+	/** The command itself */
+	CString command;
 };
 
 #endif
